# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_MeasurewiseQSchema___getitem___01():

    schema = quantizationtools.MeasurewiseQSchema()

    assert schema[0] == schema[1] == schema[2] == {
        'search_tree': quantizationtools.UnweightedSearchTree(),
        'tempo': Tempo((1, 4), 60),
        'time_signature': TimeSignature((4, 4)),
        'use_full_measure': False,
    }


def test_quantizationtools_MeasurewiseQSchema___getitem___02():

    item_a = quantizationtools.MeasurewiseQSchemaItem(search_tree=quantizationtools.UnweightedSearchTree({2: None}))
    item_b = quantizationtools.MeasurewiseQSchemaItem(tempo=((1, 4), 76))
    item_c = quantizationtools.MeasurewiseQSchemaItem(time_signature=(3, 4))
    item_d = quantizationtools.MeasurewiseQSchemaItem(
        search_tree=quantizationtools.UnweightedSearchTree({5: None}),
        use_full_measure=True
        )

    schema = quantizationtools.MeasurewiseQSchema(
        {2: item_a, 4: item_b, 7: item_c, 8: item_d},
        search_tree=quantizationtools.UnweightedSearchTree({3: None}),
        tempo=((1, 8), 58),
        time_signature=(5, 8),
        use_full_measure=False,
        )

    assert schema[0] == schema[1] == {
        'search_tree': quantizationtools.UnweightedSearchTree({3: None}),
        'tempo': Tempo((1, 8), 58),
        'time_signature': TimeSignature((5, 8)),
        'use_full_measure': False,
    }

    assert schema[2] == schema[3] == {
        'search_tree': quantizationtools.UnweightedSearchTree({2: None}),
        'tempo': Tempo((1, 8), 58),
        'time_signature': TimeSignature((5, 8)),
        'use_full_measure': False,
    }

    assert schema[4] == schema[5] == schema[6] == {
        'search_tree': quantizationtools.UnweightedSearchTree({2: None}),
        'tempo': Tempo((1, 4), 76),
        'time_signature': TimeSignature((5, 8)),
        'use_full_measure': False,
    }

    assert schema[7] == {
        'search_tree': quantizationtools.UnweightedSearchTree({2: None}),
        'tempo': Tempo((1, 4), 76),
        'time_signature': TimeSignature((3, 4)),
        'use_full_measure': False,
    }

    assert schema[8] == schema[9] == schema[1000] == {
        'search_tree': quantizationtools.UnweightedSearchTree({5: None}),
        'tempo': Tempo((1, 4), 76),
        'time_signature': TimeSignature((3, 4)),
        'use_full_measure': True,
    }



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_TonalAnalysisAgent_are_stepwise_descending_notes_01():

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    selection = tonalanalysistools.select(notes)
    assert not selection.are_stepwise_descending_notes()


def test_tonalanalysistools_TonalAnalysisAgent_are_stepwise_descending_notes_02():

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    notes.reverse()
    selection = tonalanalysistools.select(notes)
    assert selection.are_stepwise_descending_notes()



# -*- coding: utf-8 -*-
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import override


class PianoPedalSpanner(Spanner):
    r'''Piano pedal spanner.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> pedal = spannertools.PianoPedalSpanner()
            >>> attach(pedal, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \set Staff.pedalSustainStyle = #'mixed
                c'8 \sustainOn
                d'8
                e'8
                f'8 \sustainOff
            }

    Formats LilyPond ``\sustainOn``, ``\sosenutoOn`` or ``\unaCorda`` on first
    leaf in spanner.

    Formats LilyPond ``\sustainOff``, ``\sostenutoOff`` or ``\treCorde`` on
    last leaf in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_kind',
        '_style',
        )

    _kinds = {
        'sustain': (r'\sustainOn', r'\sustainOff'),
        'sostenuto': (r'\sostenutoOn', r'\sostenutoOff'),
        'corda': (r'\unaCorda', r'\treCorde'),
        }

    _styles = (
        'text',
        'bracket',
        'mixed',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        kind='sustain',
        overrides=None,
        style='mixed',
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        if not kind in list(self._kinds.keys()):
            message = 'kind must be in {!r}.'
            message = message.format(list(self._kinds.keys()))
            raise ValueError(message)
        self._kind = kind
        if not style in self._styles:
            message = 'style must be in {!r}.'
            message = message.format(self._styles)
            raise ValueError(message)
        self._style = style

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._kind = self.kind
        new._style = self.style

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import lilypondnametools
        from abjad.tools import schemetools
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        if self._is_my_only_leaf(leaf):
            style = schemetools.SchemeSymbol(self.style)
            context_setting = lilypondnametools.LilyPondContextSetting(
                context_name='Staff',
                context_property='pedalSustainStyle',
                value=style,
                )
            lilypond_format_bundle.update(context_setting)
            string = self._kinds[self.kind][0]
            lilypond_format_bundle.right.spanner_starts.append(string)
            string = self._kinds[self.kind][1]
            lilypond_format_bundle.right.spanner_starts.append(string)
        elif self._is_my_first_leaf(leaf):
            style = schemetools.SchemeSymbol(self.style)
            context_setting = lilypondnametools.LilyPondContextSetting(
                context_name='Staff',
                context_property='pedalSustainStyle',
                value=style,
                )
            lilypond_format_bundle.update(context_setting)
            string = self._kinds[self.kind][0]
            lilypond_format_bundle.right.spanner_starts.append(string)
        elif self._is_my_last_leaf(leaf):
            string = self._kinds[self.kind][1]
            lilypond_format_bundle.right.spanner_stops.append(string)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def kind(self):
        r'''Gets kind of piano pedal spanner.

        ..  container:: example

            Sustain pedal:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(kind='sustain')
                >>> attach(spanner, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set Staff.pedalSustainStyle = #'mixed
                    c'8 \sustainOn
                    d'8
                    e'8
                    f'8 \sustainOff
                }

            ::

                >>> spanner.kind
                'sustain'

        ..  container:: example

            Sostenuto pedal:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(kind='sostenuto')
                >>> attach(spanner, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set Staff.pedalSustainStyle = #'mixed
                    c'8 \sostenutoOn
                    d'8
                    e'8
                    f'8 \sostenutoOff
                }

            ::

                >>> spanner.kind
                'sostenuto'

        ..  container:: example

            Una corda / tre corde pedal:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(kind='corda')
                >>> attach(spanner, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set Staff.pedalSustainStyle = #'mixed
                    c'8 \unaCorda
                    d'8
                    e'8
                    f'8 \treCorde
                }

            ::

                >>> spanner.kind
                'corda'

        Returns ``'sustain'``, ``'sostenuto'`` or ``'corda'``.
        '''
        return self._kind

    @property
    def style(self):
        r'''Gets style of piano pedal spanner.

        ..  container:: example

            Mixed style:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(style='mixed')
                >>> attach(spanner, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set Staff.pedalSustainStyle = #'mixed
                    c'8 \sustainOn
                    d'8
                    e'8
                    f'8 \sustainOff
                }

            ::

                >>> spanner.style
                'mixed'

        ..  container:: example

            Bracket style:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(style='bracket')
                >>> attach(spanner, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set Staff.pedalSustainStyle = #'bracket
                    c'8 \sustainOn
                    d'8
                    e'8
                    f'8 \sustainOff
                }

            ::

                >>> spanner.style
                'bracket'

        ..  container:: example

            Text style:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.PianoPedalSpanner(style='text')
                >>> attach(spanner, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set Staff.pedalSustainStyle = #'text
                    c'8 \sustainOn
                    d'8
                    e'8
                    f'8 \sustainOff
                }

            ::

                >>> spanner.style
                'text'

        Returns ``'mixed'``, ``'bracket'`` or ``'text'``.
        '''
        return self._style



# -*- coding: utf-8 -*-


def greatest_common_divisor(*integers):
    '''Calculates greatest common divisor of `integers`.

    ..  container:: example

        ::

            >>> mathtools.greatest_common_divisor(84, -94, -144)
            2

    Allows nonpositive input.

    Raises not implemented error when zero is included in input.

    Returns positive integer.
    '''
    from abjad.tools import mathtools
    common_divisors = None
    for positive_integer in integers:
        all_divisors = set(mathtools.divisors(positive_integer))
        if common_divisors is None:
            common_divisors = all_divisors
        else:
            common_divisors &= all_divisors
            if common_divisors == set([1]):
                return 1
    return max(common_divisors)



# -*- coding: utf-8 -*-
from abjad import *


def test_topleveltools_set__01():
    r'''Works with score tempo interface.
    Does not include LilyPond \set command.'''

    staff = Staff("c'8 d'8 e'8 f'8")
    score = Score([staff])
    set_(score).tempo_wholes_per_minute = schemetools.SchemeMoment(24)

    assert format(score) == stringtools.normalize(
        r'''
        \new Score \with {
            tempoWholesPerMinute = #(ly:make-moment 24 1)
        } <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        >>
        '''
        )


def test_topleveltools_set__02():
    r'''Works with leaf tempo interface. Includes LilyPond \set command.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    score = Score([staff])
    moment = schemetools.SchemeMoment(24)
    leaves = select().by_leaf(flatten=True)(score)
    set_(leaves[1]).score.tempo_wholes_per_minute = moment

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'8
                \set Score.tempoWholesPerMinute = #(ly:make-moment 24 1)
                d'8
                e'8
                f'8
            }
        >>
        '''
        )


# -*- coding: utf-8 -*-
import copy
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import select
from abjad.tools.scoretools.Context import Context


class Score(Context):
    r'''A score.

    ::

        >>> staff_1 = Staff("c'8 d'8 e'8 f'8")
        >>> staff_2 = Staff("c'8 d'8 e'8 f'8")
        >>> score = Score([staff_1, staff_2])
        >>> show(score) # doctest: +SKIP

    ..  doctest::

        >>> print(format(score))
        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        >>

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Contexts'

    __slots__ = ()

    _default_context_name = 'Score'

    ### INITIALIZER ###

    def __init__(self, music=None, context_name='Score', name=None):
        Context.__init__(
            self,
            music=music,
            context_name=context_name,
            name=name,
            )
        self.is_simultaneous = True

    ### PUBLIC METHODS ###

    def add_final_bar_line(
        self, 
        abbreviation='|.',
        to_each_voice=False,
        ):
        r'''Add final bar line to end of score.


            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> score = Score([staff])

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4
                    d'4
                    e'4
                    f'4
                }
            >>

        ::

            >>> show(score) # doctest: +SKIP

        ::

            >>> score.add_final_bar_line()
            BarLine('|.')

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4
                    d'4
                    e'4
                    f'4
                    \bar "|."
                }
            >>

        ::

            >>> show(score) # doctest: +SKIP

        Set `to_each_voice` to true to make part extraction easier.

        Returns bar line.
        '''
        from abjad.tools import scoretools
        from abjad.tools import indicatortools
        double_bar = indicatortools.BarLine(abbreviation)
        if not to_each_voice:
            selection = select(self)
            last_leaf = selection._get_component(scoretools.Leaf, -1)
            attach(double_bar, last_leaf)
        else:
            for voice in iterate(self).by_class(scoretools.Voice):
                selection = select(voice)
                last_leaf = selection._get_component(scoretools.Leaf, -1)
                attach(double_bar, last_leaf)
        return double_bar

    def add_final_markup(self, markup, extra_offset=None):
        r'''Adds `markup` to end of score.

        ..  container:: example

            **Example 1.** Adds markup to last leaf:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> score = Score([staff])
                >>> place = Markup('Bremen - Boston - LA.', direction=Down)
                >>> date = Markup('July 2010 - May 2011.')
                >>> markup = Markup.right_column([place, date], direction=Down)
                >>> markup = markup.italic()
                >>> markup = score.add_final_markup(
                ...     markup,
                ...     extra_offset=(0.5, -2),
                ...     )
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(score))
                \new Score <<
                    \new Staff {
                        c'4
                        d'4
                        e'4
                        \once \override TextScript.extra-offset = #'(0.5 . -2)
                        f'4 _ \markup {
                            \italic
                                \right-column
                                    {
                                        "Bremen - Boston - LA."
                                        "July 2010 - May 2011."
                                    }
                            }
                    }
                >>

        ..  container:: example

            **Example 2.** Adds markup to last multimeasure rest:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> staff.append(MultimeasureRest(Duration(4, 4)))
                >>> score = Score([staff])
                >>> place = Markup('Bremen - Boston - LA.', direction=Down)
                >>> date = Markup('July 2010 - May 2011.')
                >>> markup = Markup.right_column([place, date], direction=Down)
                >>> markup = markup.italic()
                >>> markup = score.add_final_markup(
                ...     markup,
                ...     extra_offset=(14.5, -2),
                ...     )
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(score))
                \new Score <<
                    \new Staff {
                        c'4
                        d'4
                        e'4
                        f'4
                        \once \override MultiMeasureRestText.extra-offset = #'(14.5 . -2)
                        R1
                            _ \markup {
                                \italic
                                    \right-column
                                        {
                                            "Bremen - Boston - LA."
                                            "July 2010 - May 2011."
                                        }
                                }
                    }
                >>

        Returns none.
        '''
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        selection = select(self)
        last_leaf = selection._get_component(scoretools.Leaf, -1)
        markup = copy.copy(markup)
        attach(markup, last_leaf)
        if extra_offset is not None:
            if isinstance(last_leaf, scoretools.MultimeasureRest):
                grob_proxy = override(last_leaf).multi_measure_rest_text
            else:
                grob_proxy = override(last_leaf).text_script
            grob_proxy.extra_offset = extra_offset
        return markup



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.abctools import AbjadObject


class Job(AbjadObject):

    ### INITIALIZER ###

    def __init__(self, number):
        self.number = number

    ### SPECIAL METHODS ###

    def __call__(self):
        self.result = [x for x in mathtools.yield_all_compositions_of_integer(self.number)]

    def __repr__(self):
        return '{}({})'.format(type(self).__name__, self.number)


def test_quantizationtools_SerialJobHandler___call___01():

    jobs = [Job(x) for x in range(1, 11)]
    job_handler = quantizationtools.SerialJobHandler()
    finished_jobs = job_handler(jobs)



# -*- coding: utf-8 -*-
from abjad.tools import pitchtools
from abjad.tools.spannertools.Spanner import Spanner


class OctavationSpanner(Spanner):
    r'''Octavation spanner.

    ..  container:: example

        Spans four notes:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> spanner = spannertools.OctavationSpanner(start=1)
            >>> attach(spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \ottava #1
                c'4
                d'4
                e'4
                f'4
                \ottava #0
            }

    ..  container:: example

        Spans one note:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> octavation_spanner = spannertools.OctavationSpanner(start=1)
            >>> attach(octavation_spanner, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \ottava #1
                c'4
                \ottava #0
                d'4
                e'4
                f'4
            }

        One-note octavation changes are allowed.

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_start',
        '_stop',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        overrides=None,
        start=1,
        stop=0,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        assert isinstance(start, (int, type(None)))
        self._start = start
        assert isinstance(stop, (int, type(None)))
        self._stop = stop

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._start = self.start
        new._stop = self.stop

    def _format_after_leaf(self, leaf):
        result = []
        result.extend(Spanner._format_after_leaf(self, leaf))
        if self._is_my_last_leaf(leaf):
            string = r'\ottava #{}'.format(self.stop)
            result.append(string)
        return result

    def _format_before_leaf(self, leaf):
        result = []
        result.extend(Spanner._format_before_leaf(self, leaf))
        if self._is_my_first_leaf(leaf):
            string = r'\ottava #{}'.format(self.start)
            result.append(string)
        return result

    ### PUBLIC METHODS ###

    # TODO: add two or three more examples to better show what's going on
    def adjust_automatically(
        self,
        ottava_breakpoint=None,
        quindecisima_breakpoint=None,
        ):
        r"""Adjusts octavation spanner start and stop
        automatically according to `ottava_breakpoint`
        and `quindecisima_breakpoint`.

        ..  container:: example

            ::

                >>> measure = Measure((4, 8), "c'''8 d'''8 ef'''8 f'''8")
                >>> octavation = spannertools.OctavationSpanner()
                >>> attach(octavation, measure[:])
                >>> show(measure) # doctest: +SKIP

            ::

                >>> octavation.adjust_automatically(ottava_breakpoint=14)
                >>> show(measure) # doctest: +SKIP

            ..  doctest::

                >>> f(measure)
                    {
                        \time 4/8
                        \ottava #1
                        c'''8
                        d'''8
                        ef'''8
                        f'''8
                        \ottava #0
                    }

        Adjusts start and stop according to the diatonic pitch number of
        the maximum pitch in spanner.

        Returns none.
        """
        pitches = pitchtools.PitchSegment.from_selection(self)
        max_pitch = max(pitches)
        max_numbered_diatonic_pitch = max_pitch.diatonic_pitch_number
        if ottava_breakpoint is not None:
            if ottava_breakpoint <= max_numbered_diatonic_pitch:
                # TODO: do not adjust in place
                #       create & attach new spanner instead
                self._start = 1
                if quindecisima_breakpoint is not None:
                    if quindecisima_breakpoint <= max_numbered_diatonic_pitch:
                        self._start = 2

    ### PUBLIC PROPERTIES ###

    @property
    def start(self):
        r'''Gets octavation start.

        ..  container:: example

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> spanner = spannertools.OctavationSpanner(start=1)
                >>> attach(spanner, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> spanner.start
                1

        Returns integer or none.
        '''
        return self._start

    @property
    def stop(self):
        r'''Gets octavation stop.

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.OctavationSpanner(start=2, stop=1)
            >>> attach(spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ::

            >>> spanner.stop
            1

        Returns integer or none.
        '''
        return self._stop



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitch_max_01():
    r'''Built-in max() works when __gt__ is defined.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    written_pitches = [note.written_pitch for note in staff]
    max_pitch = max(written_pitches)

    assert max_pitch == NamedPitch('f', 4)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__comments_01():

    target = Container([Note(0, (1, 4))])

    string = r'''
    { c'4 }
    % { d'4 }
    % { e'4 }'''  # NOTE: no newline should follow the final brace

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result)



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools.TreeContainer import TreeContainer


class ReSTDocument(TreeContainer):
    r'''A ReST document tree.

    ::

        >>> document = documentationtools.ReSTDocument()
        >>> document
        ReSTDocument()

    ::

        >>> document.append(documentationtools.ReSTHeading(
        ...     level=0, text='Hello World!'))
        >>> document.append(documentationtools.ReSTParagraph(
        ...     text='blah blah blah'))
        >>> toc = documentationtools.ReSTTOCDirective()
        >>> toc.append('foo/bar')
        >>> toc.append('bar/baz')
        >>> toc.append('quux')
        >>> document.append(toc)

    ::

        >>> document
        ReSTDocument(
            children=(
                ReSTHeading(
                    level=0,
                    text='Hello World!'
                    ),
                ReSTParagraph(
                    text='blah blah blah',
                    wrap=True
                    ),
                ReSTTOCDirective(
                    children=(
                        ReSTTOCItem(
                            text='foo/bar'
                            ),
                        ReSTTOCItem(
                            text='bar/baz'
                            ),
                        ReSTTOCItem(
                            text='quux'
                            ),
                        ),
                    directive='toctree'
                    ),
                )
            )

    ::

        >>> print(document.rest_format)
        ############
        Hello World!
        ############
        <BLANKLINE>
        blah blah blah
        <BLANKLINE>
        .. toctree::
        <BLANKLINE>
           foo/bar
           bar/baz
           quux

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### PRIVATE PROPERTIES ###

    @property
    def _rest_format_contributions(self):
        result = []
        for child in self.children:
            result.extend(child._rest_format_contributions)
            result.append('')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def node_class(self):
        r'''Node class of ReST document.
        '''
        from abjad.tools import documentationtools
        return (
            documentationtools.ReSTDirective,
            documentationtools.ReSTHeading,
            documentationtools.ReSTHorizontalRule,
            documentationtools.ReSTParagraph,
            )

    @property
    def rest_format(self):
        r'''ReST format of ReST document.
        '''
        return '\n'.join(self._rest_format_contributions)



# -*- coding: utf-8 -*-
import collections
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.selectiontools.Selection import Selection


class Parentage(Selection):
    r'''A selection of components in the parentage of a component.

    ..  container:: example

        **Example.**

        ::

            >>> score = Score()
            >>> string = r"""\new Voice = "Treble Voice" { e'4 }"""
            >>> treble_staff = Staff(string, name='Treble Staff')
            >>> score.append(treble_staff)
            >>> string = r"""\new Voice = "Bass Voice" { c4 }"""
            >>> bass_staff = Staff(string, name='Bass Staff')
            >>> clef = Clef('bass')
            >>> attach(clef, bass_staff)
            >>> score.append(bass_staff)
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score <<
                \context Staff = "Treble Staff" {
                    \context Voice = "Treble Voice" {
                        e'4
                    }
                }
                \context Staff = "Bass Staff" {
                    \clef "bass"
                    \context Voice = "Bass Voice" {
                        c4
                    }
                }
            >>

        ::

            >>> bass_voice = score['Bass Voice']
            >>> note = bass_voice[0]
            >>> parentage = inspect_(note).get_parentage()

        ::

            >>> for parent in parentage:
            ...     parent
            Note('c4')
            Voice('c4')
            <Staff-"Bass Staff"{1}>
            <Score<<2>>>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_component',
        '_root',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        component=None,
        include_self=True,
        with_grace_notes=False,
        ):
        from abjad.tools import scoretools
        assert isinstance(component, (scoretools.Component, type(None)))
        if component is None:
            music = ()
        else:
            music = []
            if include_self:
                parent = component
            else:
                parent = component._parent
            while parent is not None:
                music.append(parent)
                if (
                    with_grace_notes and
                    isinstance(parent, scoretools.GraceContainer)
                    ):
                    parent = parent._carrier
                else:
                    parent = parent._parent
            music = tuple(music)
        Selection.__init__(self, music)
        self._component = component

    ### PRIVATE METHODS ###

    def _get_governor(self):
        from abjad.tools import scoretools
        for component in self:
            if isinstance(component, scoretools.Container) and \
                not component.is_simultaneous:
                if component._parent is None:
                    return component
                if isinstance(component._parent, scoretools.Container) and \
                    component._parent.is_simultaneous:
                    return component

    @staticmethod
    def _id_string(component):
        lhs = component.__class__.__name__
        rhs = getattr(component, 'name', None) or id(component)
        return '{}-{!r}'.format(lhs, rhs)

    ### PUBLIC METHODS ###

    def get_first(self, prototype=None):
        r'''Gets first instance of `prototype` in parentage.

        Returns component or none.
        '''
        from abjad.tools import scoretools
        if prototype is None:
            prototype = (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        for component in self:
            if isinstance(component, prototype):
                return component

    ### PRIVATE PROPERTIES ###

    @property
    def _prolations(self):
        prolations = []
        default = durationtools.Multiplier(1)
        for parent in self:
            prolation = getattr(parent, 'implied_prolation', default)
            prolations.append(prolation)
        return prolations

    ### PUBLIC PROPERTIES ###

    @property
    def component(self):
        r'''The component from which the selection was derived.

        Returns component.
        '''
        return self._component

    @property
    def depth(self):
        r'''Length of proper parentage of component.

        Returns nonnegative integer.
        '''
        return len(self[1:])

    @property
    def is_grace_note(self):
        r'''Is true when parentage contains a grace container.
        Otherwise false.

        .. container:: example

            **Example.** Grace notes:

            ::

                >>> voice = Voice("c'4 d'4 e'4 f'4")
                >>> grace_notes = [Note("c'16"), Note("d'16")]
                >>> grace_container = scoretools.GraceContainer(
                ...     grace_notes,
                ...     kind='grace',
                ...     )
                >>> attach(grace_container, voice[1])
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    c'4
                    \grace {
                        c'16
                        d'16
                    }
                    d'4
                    e'4
                    f'4
                }

            ::

                >>> for leaf in iterate(voice).by_leaf(with_grace_notes=True):
                ...     parentage = inspect_(leaf).get_parentage()
                ...     print(leaf, parentage.is_grace_note)
                c'4 False
                c'16 True
                d'16 True
                d'4 False
                e'4 False
                f'4 False

        Returns true or false.
        '''
        from abjad.tools import scoretools
        grace_container = self.get_first(prototype=scoretools.GraceContainer)
        if grace_container is not None:
            return True
        return False

    @property
    def is_orphan(self):
        r'''Is true when component has no parent.
        Otherwise false.

        Returns true or false.
        '''
        return self.parent is None

    @property
    def logical_voice(self):
        r'''Gets logical voice.

        ..  container:: example

            **Example.** Gets logical voice of note:

            ::

                >>> voice = Voice("c'4 d'4 e'4 f'4", name='CustomVoice')
                >>> staff = Staff([voice], name='CustomStaff')
                >>> score = Score([staff], name='CustomScore')
                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \context Score = "CustomScore" <<
                    \context Staff = "CustomStaff" {
                        \context Voice = "CustomVoice" {
                            c'4
                            d'4
                            e'4
                            f'4
                        }
                    }
                >>

            ::

                >>> note = voice[0]
                >>> parentage = inspect_(note).get_parentage()
                >>> logical_voice = parentage.logical_voice

            ::

                >>> for key, value in logical_voice.items():
                ...     print('%12s: %s' % (key, value))
                ...
                score: Score-'CustomScore'
                staff group:
                staff: Staff-'CustomStaff'
                voice: Voice-'CustomVoice'

        Returns ordered dictionary.
        '''
        from abjad.tools import scoretools
        keys = ('score', 'staff group', 'staff', 'voice')
        logical_voice = collections.OrderedDict.fromkeys(keys, '')
        for component in self:
            if isinstance(component, scoretools.Voice):
                if not logical_voice['voice']:
                    logical_voice['voice'] = self._id_string(component)
            elif isinstance(component, scoretools.Staff):
                if not logical_voice['staff']:
                    logical_voice['staff'] = self._id_string(component)
                    # explicit staff demands a nested voice:
                    # if no explicit voice has been found,
                    # create implicit voice here with random integer
                    if not logical_voice['voice']:
                        logical_voice['voice'] = id(component)
            elif isinstance(component, scoretools.StaffGroup):
                if not logical_voice['staff group']:
                    logical_voice['staff group'] = self._id_string(component)
            elif isinstance(component, scoretools.Score):
                if not logical_voice['score']:
                    logical_voice['score'] = self._id_string(component)
        return logical_voice

    @property
    def parent(self):
        r'''Gets parent.

        Returns none when component has no parent.

        Returns component or none.
        '''
        if 1 < len(self):
            return self[1]

    @property
    def prolation(self):
        r'''Gets prolation.

        Returns multiplier.
        '''
        prolations = [durationtools.Multiplier(1)] + self._prolations
        products = mathtools.cumulative_products(prolations)
        return products[-1]

    @property
    def root(self):
        r'''Gets root.

        Root defined equal to last component in parentage.

        Returns component.
        '''
        return self[-1]

    @property
    def score_index(self):
        r'''Gets score index.

        ..  todo:: Define score index for grace notes.

        ..  container:: example

            **Example.** Gets note score indices:

            ::

                >>> staff_1 = Staff(r"\times 2/3 { c''2 b'2 a'2 }")
                >>> staff_2 = Staff("c'2 d'2")
                >>> score = Score([staff_1, staff_2])
                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score <<
                    \new Staff {
                        \times 2/3 {
                            c''2
                            b'2
                            a'2
                        }
                    }
                    \new Staff {
                            c'2
                            d'2
                    }
                >>

            ::

                >>> selector = select().by_leaf(flatten=True)
                >>> leaves = selector(score)
                >>> for leaf in leaves:
                ...     parentage = inspect_(leaf).get_parentage()
                ...     leaf, parentage.score_index
                ...
                (Note("c''2"), (0, 0, 0))
                (Note("b'2"), (0, 0, 1))
                (Note("a'2"), (0, 0, 2))
                (Note("c'2"), (1, 0))
                (Note("d'2"), (1, 1))

        ..  container:: example

            **Example 1.** With grace notes:

            ::

                >>> voice = Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> grace_notes = [Note("cf''16"), Note("bf'16")]
                >>> grace = scoretools.GraceContainer(
                ...     grace_notes,
                ...     kind='grace',
                ...     )
                >>> attach(grace, voice[1])
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    c'8 [
                    \grace {
                        cf''16
                        bf'16
                    }
                    d'8
                    e'8
                    f'8 ]
                }

            ::

                >>> leaves = iterate(voice).by_class(with_grace_notes=True)
                >>> for leaf in leaves:
                ...     parentage = inspect_(leaf).get_parentage()
                ...     leaf, parentage.score_index
                ...
                (Voice("c'8 d'8 e'8 f'8"), ())
                (Note("c'8"), (0,))
                (Note("cf''16"), (0,))
                (Note("bf'16"), (1,))
                (Note("d'8"), (1,))
                (Note("e'8"), (2,))
                (Note("f'8"), (3,))

            ..  todo:: Incorrect values returned for grace notes.

        Returns tuple of zero or more nonnegative integers.
        '''
        result = []
        current = self[0]
        for parent in self[1:]:
            index = parent.index(current)
            result.insert(0, index)
            current = parent
        result = tuple(result)
        return result

    @property
    def tuplet_depth(self):
        r'''Gets tuplet depth.

        ..  container:: example

            **Example.** Gets tuplet depth:

            ::

                >>> tuplet = Tuplet(Multiplier(2, 3), "c'2 d'2 e'2")
                >>> staff = Staff([tuplet])
                >>> note = tuplet[0]
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \times 2/3 {
                        c'2
                        d'2
                        e'2
                    }
                }

            ::

                >>> inspect_(note).get_parentage().tuplet_depth
                1

            ::

                >>> inspect_(tuplet).get_parentage().tuplet_depth
                0

            ::

                >>> inspect_(staff).get_parentage().tuplet_depth
                0

        Returns nonnegative integer.
        '''
        from abjad.tools import scoretools
        result = 0
        # should probably interate up to only first simultaneous container
        # in parentage.
        # note that we probably need a named idea for 'parentage
        # up to first simultaneous container'.
        for parent in self[1:]:
            if isinstance(parent, scoretools.Tuplet):
                result += 1
        return result



# -*- coding: utf-8 -*-
import collections
from abjad.tools import systemtools
from abjad.tools.datastructuretools.TypedCollection import TypedCollection


class TypedOrderedDict(TypedCollection):
    r'''A typed ordered dictionary.

    ..  container:: example

        **Example 1.** Initializes from list of pairs:

        ::

            >>> dictionary = datastructuretools.TypedOrderedDict([
            ...     ('color', 'red'),
            ...     ('directive', Markup(r'\italic Allegretto')),
            ...     ])

        ::

            >>> print(format(dictionary))
            abjad.datastructuretools.TypedOrderedDict(
                [
                    ('color', 'red'),
                    (
                        'directive',
                        markuptools.Markup(
                            contents=[
                                markuptools.MarkupCommand(
                                    'italic',
                                    'Allegretto'
                                    ),
                                ],
                            ),
                        ),
                    ]
                )

    ..  container:: example

        **Example 2.** Initializes from built-in dictionary:

        ::

            >>> dictionary = {
            ...     'color': 'red',
            ...     'directive': Markup(r'\italic Allegretto'),
            ...     }
            >>> dictionary = datastructuretools.TypedOrderedDict(
            ...     dictionary
            ...     )

        ::

            >>> print(format(dictionary))
            abjad.datastructuretools.TypedOrderedDict(
                [
                    ('color', 'red'),
                    (
                        'directive',
                        markuptools.Markup(
                            contents=[
                                markuptools.MarkupCommand(
                                    'italic',
                                    'Allegretto'
                                    ),
                                ],
                            ),
                        ),
                    ]
                )

    ..  container:: example

        **Example 3.** Initializes from other typed ordered dictionary:

        ::

            >>> dictionary_1 = datastructuretools.TypedOrderedDict([
            ...     ('color', 'red'),
            ...     ('directive', Markup(r'\italic Allegretto')),
            ...     ])
            >>> dictionary_2 = datastructuretools.TypedOrderedDict(
            ...     dictionary_1
            ...     )

        ::

            >>> print(format(dictionary_2))
            abjad.datastructuretools.TypedOrderedDict(
                [
                    ('color', 'red'),
                    (
                        'directive',
                        markuptools.Markup(
                            contents=[
                                markuptools.MarkupCommand(
                                    'italic',
                                    'Allegretto'
                                    ),
                                ],
                            ),
                        ),
                    ]
                )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        TypedCollection.__init__(
            self,
            item_class=item_class,
            )
        if isinstance(items, dict):
            items = sorted(items.items())
        elif isinstance(items, collections.Mapping):
            items = list(items.items())
        items = items or []
        the_items = []
        for item in items:
            assert len(item) == 2, repr(item)
            key = item[0]
            value = self._item_coercer(item[1])
            the_item = (key, value)
            the_items.append(the_item)
        self._collection = collections.OrderedDict(the_items)

    ### SPECIAL METHODS ###

    def __cmp__(self, argument):
        r'''Aliases OrderedDict.__cmp__().

        Returns true or false.
        '''
        assert isinstance(argument, type(self))
        ordered_dictionary = argument._collection
        return self._collection.__cmp__(ordered_dictionary)

    def __contains__(self, key):
        r'''Aliases OrderedDict.__contains__().

        Returns true or false.
        '''
        return key in self._collection

    def __delitem__(self, i):
        r'''Aliases OrderedDict.__delitem__().

        Returns none.
        '''
        del(self._collection[i])

    def __ge__(self, argument):
        r'''Is true when typed ordered dictionary is greater than or equal
        to `argument`. Otherwise false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.__ge__(argument._collection)

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        Returns item or slice.
        '''
        return self._collection.__getitem__(argument)

    def __gt__(self, argument):
        r'''Is true when typed ordered dictionary is greater than `argument`.
        Otherwise false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.__gt__(argument._collection)

    def __le__(self, argument):
        r'''Is true when typed ordered dictionary is less than or equal
        to `argument`. Otherwise false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.__le__(argument._collection)

    def __lt__(self, argument):
        r'''Is true when typed ordered dictionary is less than `argument`.
        Otherwise false.

        Returns true or false.
        '''
        argument = type(self)(argument)
        return self._collection.__lt__(argument._collection)

    def __ne__(self, argument):
        r'''Is true when typed ordered dictionary is not equal to `argument`.
        Otherwise false.

        Returns true or false.
        '''
        return not self == argument

    def __reversed__(self):
        r'''Aliases OrderedDict.__reversed__().

        Returns generatos.
        '''
        return self._collection.__reversed__()

    def __setitem__(self, i, argument):
        r'''Changes items in `argument` to items and sets.

        Returns none.
        '''
        new_item = self._item_coercer(argument)
        self._collection[i] = new_item

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        if 'items' in names:
            names.remove('items')
        values = [list(self._collection.items())]
        return systemtools.FormatSpecification(
            self,
            repr_is_indented=False,
            storage_format_args_values=values,
            storage_format_kwargs_names=names,
            storage_format_includes_root_package=True,
            )

    ### PUBLIC METHODS ###

    def clear(self):
        r'''Clears typed ordered dictionary.

        Returns none.
        '''
        self._collection.clear()

    def copy(self):
        r'''Copies typed ordered dictionary.

        Returns new typed ordered dictionary.
        '''
        ordered_dictionary = self._collection.copy()
        items = list(ordered_dictionary.items())
        return type(self)(
            item_class=self.item_class,
            items=items,
            )

    def get(self, i, default=None):
        r'''Aliases OrderedDict.get().

        Returns item or raises key error.
        '''
        return self._collection.get(i, default)

    def has_key(self, key):
        r'''Aliases OrderdDict.has_key().

        Returns true or false.
        '''
        return key in self._collection

    def items(self):
        r'''Aliases OrderedDict.items().

        Returns generator.
        '''
        return iter(self._collection.items())

    def keys(self):
        r'''Aliases OrderedDict.keys().

        Returns generator.
        '''
        return iter(self._collection.keys())

    def pop(self, key, default=None):
        r'''Aliases OrderedDict.pop().

        Returns items.
        '''
        return self._collection.pop(key, default)

    def popitem(self):
        r'''Aliases OrderedDict.popitem().

        Returns generator.
        '''
        return self._collection.popitem()

    def setdefault(self, key, default=None):
        r'''Aliases OrderedDict.setdefault().

        Returns items.
        '''
        return self._collection.setdefault(key, default)

    def update(self, *arguments, **keywords):
        r'''Aliases OrderedDict.update().

        Returns none.
        '''
        return self._collection.update(*arguments, **keywords)

    def values(self):
        r'''Aliases OrderedDict.values().

        Returns generator.
        '''
        return iter(self._collection.values())


collections.MutableMapping.register(TypedOrderedDict)



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class KeySignature(AbjadValueObject):
    r'''Key signature.

    ..  container:: example

        **Example 1.** E major:

        ::

            >>> staff = Staff("e'8 fs'8 gs'8 a'8")
            >>> key_signature = KeySignature('e', 'major')
            >>> attach(key_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \key e \major
                e'8
                fs'8
                gs'8
                a'8
            }

    ..  container:: example

        **Example 2.** e minor:

        ::

            >>> staff = Staff("e'8 fs'8 g'8 a'8")
            >>> key_signature = KeySignature('e', 'minor')
            >>> attach(key_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \key e \minor
                e'8
                fs'8
                g'8
                a'8
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_mode',
        '_tonic',
        )

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, tonic='c', mode='major'):
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools import tonalanalysistools
        tonic = pitchtools.NamedPitchClass(tonic)
        mode = tonalanalysistools.Mode(mode)
        self._tonic = tonic
        self._mode = mode
        self._default_scope = scoretools.Staff

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of key signature.

        ..  container:: example

            **Example 1.** E major:

            ::

                >>> str(KeySignature('e', 'major'))
                'e-major'

        ..  container:: example

            **Example 2.** e minor:

            ::

                >>> str(KeySignature('e', 'minor'))
                'e-minor'

        Returns string.
        '''
        return '{!s}-{!s}'.format(self.tonic, self.mode)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return '{!r}, {!r}'.format(self.tonic, self.mode)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return r'\key {!s} \{!s}'.format(self.tonic, self.mode)

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of key signature.

        ..  container:: example

            **Example 1.** E major:

            ::

                >>> key_signature = KeySignature('e', 'major')
                >>> key_signature.default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        ..  container:: example

            **Example 2.** e minor:

            ::

                >>> key_signature = KeySignature('e', 'minor')
                >>> key_signature.default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        Key signatures are staff-scoped by default.

        Returns staff.
        '''
        return self._default_scope

    @property
    def mode(self):
        r'''Gets mode of key signature.

        ..  container:: example

            **Example 1.** E major:

            ::

                >>> key_signature = KeySignature('e', 'major')
                >>> key_signature.mode
                Mode(mode_name='major')

        ..  container:: example

            **Example 2.** e minor:

            ::

                >>> key_signature = KeySignature('e', 'minor')
                >>> key_signature.mode
                Mode(mode_name='minor')

        Returns mode.
        '''
        return self._mode

    @property
    def name(self):
        r'''Gets name of key signature.

        ..  container:: example

            **Example 1.** E major:

            ::

                >>> key_signature = KeySignature('e', 'major')
                >>> key_signature.name
                'E major'

        ..  container:: example

            **Example 2.** e minor:

            ::

                >>> key_signature = KeySignature('e', 'minor')
                >>> key_signature.name
                'e minor'

        Returns string.
        '''
        if self.mode.mode_name == 'major':
            tonic = str(self.tonic).upper()
        else:
            tonic = str(self.tonic)
        return '{!s} {!s}'.format(tonic, self.mode.mode_name)

    @property
    def tonic(self):
        r'''Gets tonic of key signature.

        ..  container:: example

            **Example 1.** E major:

            ::

                >>> key_signature = KeySignature('e', 'major')
                >>> key_signature.tonic
                NamedPitchClass('e')

        ..  container:: example

            **Example 2.** e minor:

            ::

                >>> key_signature = KeySignature('e', 'minor')
                >>> key_signature.tonic
                NamedPitchClass('e')

        Returns named pitch-class.
        '''
        return self._tonic



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_RomanNumeral___eq___01():

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'dominant', 7, 0)
    u = tonalanalysistools.RomanNumeral(5, 'dominant', 7, 0, (4, 3))
    voice = tonalanalysistools.RomanNumeral(5, 'dominant', 7, 0, (4, 3))

    assert      roman_numeral == roman_numeral
    assert not roman_numeral == u
    assert not roman_numeral == voice

    assert not u == roman_numeral
    assert      u == u
    assert      u == voice

    assert not voice == roman_numeral
    assert      voice == u
    assert      voice == voice



# -*- coding: utf-8 -*-
import copy
from abjad.tools import systemtools
from abjad.tools.scoretools.Container import Container


class Context(Container):
    '''A horizontal layer of music.

    ::

        >>> context = scoretools.Context(
        ...     name='MeterVoice',
        ...     context_name='TimeSignatureContext',
        ...     )

    ::

        >>> context
        Context()

    ..  doctest::

        >>> print(format(context))
        \context TimeSignatureContext = "MeterVoice" {
        }


    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Contexts'

    __slots__ = (
        '_context_name',
        '_consists_commands',
        '_remove_commands',
        '_is_nonsemantic',
        )

    _default_context_name = 'Voice'

    ### INITIALIZER ###

    def __init__(
        self,
        music=None,
        context_name='Context',
        is_simultaneous=None,
        name=None,
        ):
        Container.__init__(
            self,
            is_simultaneous=is_simultaneous,
            music=music,
            name=name,
            )
        self.context_name = context_name
        self._consists_commands = []
        self._remove_commands = []
        self.is_nonsemantic = False

    ### SPECIAL METHODS ###

    def __repr__(self):
        r'''Gets interpreter representation of context.

        ::

            >>> context = scoretools.Context(
            ...     name='MeterVoice',
            ...     context_name='TimeSignatureContext',
            ...     )
            >>> repr(context)
            'Context()'

        Returns string.
        '''
        from abjad.tools import scoretools
        if all(isinstance(x, scoretools.Leaf) for x in self):
            return Container.__repr__(self)
        return self._get_abbreviated_string_format()

    ### PRIVATE METHODS ###

    def _copy_with_indicators_but_without_children_or_spanners(self):
        new = Container._copy_with_indicators_but_without_children_or_spanners(self)
        new._consists_commands = copy.copy(self.consists_commands)
        new._remove_commands = copy.copy(self.remove_commands)
        new.context_name = copy.copy(self.context_name)
        new.name = copy.copy(self.name)
        new.is_nonsemantic = copy.copy(self.is_nonsemantic)
        return new

    def _format_closing_slot(context, bundle):
        result = []
        result.append(('indicators', bundle.closing.indicators))
        result.append(('commands', bundle.closing.commands))
        result.append(('comments', bundle.closing.comments))
        return context._format_slot_contributions_with_indent(result)

    def _format_consists_commands(self):
        result = []
        for engraver in self.consists_commands:
            string = r'\consists {}'.format(engraver)
            result.append(string)
        return result

    def _format_invocation(self):
        if self.name is not None:
            string = r'\context {} = "{}"'
            string = string.format(self.context_name, self.name)
        else:
            string = r'\new {}'
            string = string.format(self.context_name)
        return string

    def _format_open_brackets_slot(context, bundle):
        indent = systemtools.LilyPondFormatManager.indent
        result = []
        if context.is_simultaneous:
            brackets_open = ['<<']
        else:
            brackets_open = ['{']
        remove_commands = context._format_remove_commands()
        consists_commands = context._format_consists_commands()
        overrides = bundle.grob_overrides
        settings = bundle.context_settings
        if remove_commands or consists_commands or overrides or settings:
            contributions = [context._format_invocation() + r' \with {']
            contributions = tuple(contributions)
            identifier_pair = ('context_brackets', 'open')
            result.append((identifier_pair, contributions))
            contributions = [indent + x for x in remove_commands]
            contributions = tuple(contributions)
            identifier_pair = ('engraver removals', 'remove_commands')
            result.append((identifier_pair, contributions))
            contributions = [indent + x for x in consists_commands]
            contributions = tuple(contributions)
            identifier_pair = ('engraver consists', 'consists_commands')
            result.append((identifier_pair, contributions))
            contributions = [indent + x for x in overrides]
            contributions = tuple(contributions)
            identifier_pair = ('overrides', 'overrides')
            result.append((identifier_pair, contributions))
            contributions = [indent + x for x in settings]
            contributions = tuple(contributions)
            identifier_pair = ('settings', 'settings')
            result.append((identifier_pair, contributions))
            contributions = ['}} {}'.format(brackets_open[0])]
            contributions = tuple(contributions)
            identifier_pair = ('context_brackets', 'open')
            result.append((identifier_pair, contributions))
        else:
            contribution = context._format_invocation()
            contribution += ' {}'.format(brackets_open[0])
            contributions = [contribution]
            contributions = tuple(contributions)
            identifier_pair = ('context_brackets', 'open')
            result.append((identifier_pair, contributions))
        return tuple(result)

    def _format_opening_slot(context, bundle):
        result = []
        result.append(('comments', bundle.opening.comments))
        result.append(('indicators', bundle.opening.indicators))
        result.append(('commands', bundle.opening.commands))
        return context._format_slot_contributions_with_indent(result)

    def _format_remove_commands(self):
        result = []
        for engraver in self.remove_commands:
            string = r'\remove {}'.format(engraver)
            result.append(string)
        return result

    def _get_format_pieces(self):
        return self._format_component(pieces=True)

    def _get_lilypond_format(self):
        self._update_now(indicators=True)
        return self._format_component()

    ### PUBLIC PROPERTIES ###

    @property
    def consists_commands(self):
        r'''Unordered set of LilyPond engravers to include
        in context definition.

        Manage with add, update, other standard set commands:

        ::

            >>> staff = Staff([])
            >>> staff.consists_commands.append('Horizontal_bracket_engraver')
            >>> print(format(staff))
            \new Staff \with {
                \consists Horizontal_bracket_engraver
            } {
            }

        '''
        return self._consists_commands

    @property
    def context_name(self):
        r'''Gets and sets context name of context.

        Returns string.
        '''
        return self._context_name

    @context_name.setter
    def context_name(self, argument):
        if argument is None:
            argument = type(self).__name__
        else:
            argument = str(argument)
        self._context_name = argument

    @property
    def is_nonsemantic(self):
        r'''Gets and sets nonsemantic voice flag.

        ::

            >>> pairs = [(1, 8), (5, 16), (5, 16)]
            >>> measures = scoretools.make_spacer_skip_measures(pairs)
            >>> voice = Voice(measures)
            >>> voice.name = 'HiddenTimeSignatureVoice'

        ::

            >>> voice.is_nonsemantic = True

        ..  doctest::

            >>> print(format(voice))
            \context Voice = "HiddenTimeSignatureVoice" {
                {
                    \time 1/8
                    s1 * 1/8
                }
                {
                    \time 5/16
                    s1 * 5/16
                }
                {
                    s1 * 5/16
                }
            }

        ::

            >>> voice.is_nonsemantic
            True

        Gets nonsemantic voice voice:

        ::

            >>> voice = Voice([])

        ::

            >>> voice.is_nonsemantic
            False

        Returns true or false.

        The intent of this read / write attribute is to allow composers
        to tag invisible voices used to house time signatures indications,
        bar number directives or other pieces of score-global non-musical
        information. Such nonsemantic voices can then be omitted from
        voice interation and other functions.
        '''
        return self._is_nonsemantic

    @is_nonsemantic.setter
    def is_nonsemantic(self, arg):
        if not isinstance(arg, bool):
            raise TypeError
        self._is_nonsemantic = arg

    @property
    def is_semantic(self):
        r'''Is true when context is semantic. Otherwise false.

        Returns true or false.
        '''
        return not self.is_nonsemantic

    @property
    def lilypond_context(self):
        r'''Gets `LilyPondContext` associated with context.

        Returns LilyPond context instance.
        '''
        from abjad.tools import lilypondnametools
        try:
            lilypond_context = lilypondnametools.LilyPondContext(
                name=self.context_name,
                )
        except AssertionError:
            lilypond_context = lilypondnametools.LilyPondContext(
                name=self._default_context_name,
                )
        return lilypond_context

    @property
    def remove_commands(self):
        r'''Unordered set of LilyPond engravers to remove from context.

        Manage with add, update, other standard set commands:

        ::

            >>> staff = Staff([])
            >>> staff.remove_commands.append('Time_signature_engraver')
            >>> print(format(staff))
            \new Staff \with {
                \remove Time_signature_engraver
            } {
            }

        '''
        return self._remove_commands



# -*- coding: utf-8 -*-
from abjad.tools import metertools
from abjad.tools import scoretools
from abjad.tools.topleveltools import mutate
from abjad.tools.quantizationtools.AttackPointOptimizer \
    import AttackPointOptimizer


class MeasurewiseAttackPointOptimizer(AttackPointOptimizer):
    r'''Concrete ``AttackPointOptimizer`` instance which attempts to optimize
    attack points in an expression with regard to the effective time signature
    of that expression.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
        >>> show(staff) # doctest: +SKIP

    ::

        >>> source_tempo = Tempo((1, 4), 60)
        >>> q_events = quantizationtools.QEventSequence.from_tempo_scaled_leaves(
        ...     staff[:],
        ...     tempo=source_tempo,
        ...     )
        >>> target_tempo = Tempo((1, 4), 54)
        >>> q_schema = quantizationtools.MeasurewiseQSchema(
        ...     tempo=target_tempo,
        ...     )
        >>> quantizer = quantizationtools.Quantizer()

    Without the measure-wise attack-point optimizer:

    ::

        >>> result = quantizer(
        ...     q_events,
        ...     q_schema=q_schema,
        ...     )
        >>> show(result) # doctest: +SKIP

    With the measure-wise attack-point optimizer:

    ::

        >>> optimizer = quantizationtools.MeasurewiseAttackPointOptimizer()
        >>> result = quantizer(
        ...     q_events,
        ...     attack_point_optimizer=optimizer,
        ...     q_schema=q_schema,
        ...     )
        >>> show(result) # doctest: +SKIP

    Only acts on Measure instances.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, argument):
        r'''Calls measurewise attack-point optimizer.

        Returns none.
        '''
        assert isinstance(argument, scoretools.Measure)
        meter = metertools.Meter(argument)
        mutate(argument[:]).rewrite_meter(
            meter,
            boundary_depth=1,
            )


# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import lilypondnametools
from abjad.tools import schemetools
from abjad.tools import scoretools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import inspect_


class BowContactSpanner(Spanner):
    r'''Bow contact spanner.

    ..  container:: example

        ::

            >>> staff = Staff()
            >>> staff.extend(r"c'4. c'8 \times 2/3 { c'4 c'4 c'4 }")

        ::

            >>> selector = select().by_leaf(flatten=True)
            >>> leaves = selector(staff)
            >>> attach(indicatortools.BowMotionTechnique('jete'), leaves[0])
            >>> attach(indicatortools.BowContactPoint((1, 4)), leaves[0])
            >>> attach(indicatortools.BowContactPoint((3, 4)), leaves[1])
            >>> attach(indicatortools.BowContactPoint((1, 2)), leaves[2])
            >>> attach(indicatortools.BowMotionTechnique('circular'),
            ...     leaves[3])
            >>> attach(indicatortools.BowContactPoint((1, 1)), leaves[3])
            >>> attach(indicatortools.BowContactPoint((0, 1)), leaves[4])

        ::

            >>> attach(Clef('percussion'), staff)
            >>> override(staff).bar_line.transparent = True
            >>> override(staff).dots.staff_position = -8
            >>> override(staff).flag.Y_offset = -8.5
            >>> override(staff).glissando.bound_details__left__padding = 1.5
            >>> override(staff).glissando.bound_details__right__padding = 1.5
            >>> override(staff).glissando.thickness = 2
            >>> override(staff).script.staff_padding = 3
            >>> override(staff).staff_symbol.transparent = True
            >>> override(staff).stem.direction = Down
            >>> override(staff).stem.length = 8
            >>> override(staff).stem.stem_begin_position = -9
            >>> override(staff).time_signature.stencil = False

        ::

            >>> attach(spannertools.BowContactSpanner(), leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                \override BarLine.transparent = ##t
                \override Dots.staff-position = #-8
                \override Flag.Y-offset = #-8.5
                \override Glissando.bound-details.left.padding = #1.5
                \override Glissando.bound-details.right.padding = #1.5
                \override Glissando.thickness = #2
                \override Script.staff-padding = #3
                \override StaffSymbol.transparent = ##t
                \override Stem.direction = #down
                \override Stem.length = #8
                \override Stem.stem-begin-position = #-9
                \override TimeSignature.stencil = ##f
            } {
                \clef "percussion"
                \once \override Glissando.style = #'dotted-line
                \once \override NoteHead.Y-offset = -1.0
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \center-align
                        \vcenter
                            \fraction
                                1
                                4
                    }
                c'4. ^\downbow \glissando
                \once \override NoteHead.Y-offset = 1.0
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \center-align
                        \vcenter
                            \fraction
                                3
                                4
                    }
                c'8 ^\upbow \glissando
                \times 2/3 {
                    \once \override NoteHead.Y-offset = 0.0
                    \once \override NoteHead.stencil = #ly:text-interface::print
                    \once \override NoteHead.text = \markup {
                        \center-align
                            \vcenter
                                \fraction
                                    1
                                    2
                        }
                    c'4 ^\downbow \glissando
                    \once \override Glissando.style = #'zigzag
                    \once \override NoteHead.Y-offset = 2.0
                    \once \override NoteHead.stencil = #ly:text-interface::print
                    \once \override NoteHead.text = \markup {
                        \center-align
                            \vcenter
                                \fraction
                                    1
                                    1
                        }
                    c'4 ^\upbow \glissando
                    \once \override NoteHead.Y-offset = -2.0
                    \once \override NoteHead.stencil = #ly:text-interface::print
                    \once \override NoteHead.text = \markup {
                        \center-align
                            \vcenter
                                \fraction
                                    0
                                    1
                        }
                    c'4
                }
            }

    ..  container:: example

        Use ``BowContactPoint(None)`` to indicate un-bowed actions, such as
        pizzicato.

        ::

            >>> staff = Staff()
            >>> staff.extend(r"c'4 c'4 c'4 c'4")

        ::

            >>> leaves = staff[:]
            >>> attach(indicatortools.BowContactPoint(None), leaves[0])
            >>> attach(indicatortools.BowContactPoint((3, 4)), leaves[1])
            >>> attach(indicatortools.BowContactPoint((1, 2)), leaves[2])
            >>> attach(indicatortools.BowContactPoint(None), leaves[3])

        ::

            >>> attach(Clef('percussion'), staff)
            >>> override(staff).bar_line.transparent = True
            >>> override(staff).dots.staff_position = -8
            >>> override(staff).flag.Y_offset = -8.5
            >>> override(staff).glissando.bound_details__left__padding = 1.5
            >>> override(staff).glissando.bound_details__right__padding = 1.5
            >>> override(staff).glissando.thickness = 2
            >>> override(staff).script.staff_padding = 3
            >>> override(staff).staff_symbol.transparent = True
            >>> override(staff).stem.direction = Down
            >>> override(staff).stem.length = 8
            >>> override(staff).stem.stem_begin_position = -9
            >>> override(staff).time_signature.stencil = False

        ::

            >>> attach(spannertools.BowContactSpanner(), leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                \override BarLine.transparent = ##t
                \override Dots.staff-position = #-8
                \override Flag.Y-offset = #-8.5
                \override Glissando.bound-details.left.padding = #1.5
                \override Glissando.bound-details.right.padding = #1.5
                \override Glissando.thickness = #2
                \override Script.staff-padding = #3
                \override StaffSymbol.transparent = ##t
                \override Stem.direction = #down
                \override Stem.length = #8
                \override Stem.stem-begin-position = #-9
                \override TimeSignature.stencil = ##f
            } {
                \clef "percussion"
                \once \override NoteHead.style = #'cross
                c'4
                \once \override NoteHead.Y-offset = 1.0
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \center-align
                        \vcenter
                            \fraction
                                3
                                4
                    }
                c'4 ^\upbow \glissando
                \once \override NoteHead.Y-offset = 0.0
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \center-align
                        \vcenter
                            \fraction
                                1
                                2
                    }
                c'4
                \once \override NoteHead.style = #'cross
                c'4
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, overrides=None):
        Spanner.__init__(self, overrides=overrides)

    ### PRIVATE METHODS ###

    def _get_annotations(self, leaf):
        inspector = inspect_(leaf)
        bow_contact_point = None
        prototype = indicatortools.BowContactPoint
        if inspector.has_indicator(prototype):
            bow_contact_point = inspector.get_indicator(prototype)
        bow_motion_technique = None
        prototype = indicatortools.BowMotionTechnique
        if inspector.has_indicator(prototype):
            bow_motion_technique = inspector.get_indicator(prototype)
        return (
            bow_contact_point,
            bow_motion_technique,
            )

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        indicators = self._get_annotations(leaf)
        bow_contact_point = indicators[0]
        bow_motion_technique = indicators[1]
        #print(leaf)
        if bow_contact_point is None:
            #print('\t', None)
            return lilypond_format_bundle
        if bow_contact_point.contact_point is None:
            #print('\t', 'PIZZ')
            self._make_pizzicato_overrides(lilypond_format_bundle)
            return lilypond_format_bundle
        if self._is_my_only_leaf(leaf):
            #print('\t', 'ONLY')
            return lilypond_format_bundle
        #print('\t', 'NORM')
        self._make_bow_contact_point_overrides(
            bow_contact_point=bow_contact_point,
            lilypond_format_bundle=lilypond_format_bundle,
            )
        if self._next_leaf_is_bowed(leaf):
            lilypond_format_bundle.right.spanner_starts.append(r'\glissando')
            self._make_bow_direction_change_contributions(
                bow_contact_point=bow_contact_point,
                leaf=leaf,
                lilypond_format_bundle=lilypond_format_bundle,
                )
            self._make_glissando_overrides(
                bow_motion_technique=bow_motion_technique,
                lilypond_format_bundle=lilypond_format_bundle,
                )
        return lilypond_format_bundle

    def _make_bow_contact_point_overrides(
        self,
        bow_contact_point=None,
        lilypond_format_bundle=None,
        ):
        if bow_contact_point is None:
            return
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='NoteHead',
            is_once=True,
            property_path='stencil',
            value=schemetools.Scheme('ly:text-interface::print'),
            )
        string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(string)
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='NoteHead',
            is_once=True,
            property_path='text',
            value=bow_contact_point.markup,
            )
        string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(string)
        y_offset = float((4 * bow_contact_point.contact_point) - 2)
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='NoteHead',
            is_once=True,
            property_path='Y-offset',
            value=y_offset,
            )
        string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(string)

    def _make_bow_direction_change_contributions(
        self,
        bow_contact_point=None,
        leaf=None,
        lilypond_format_bundle=None,
        ):
        cautionary_change = False
        direction_change = None
        next_leaf = inspect_(leaf).get_leaf(1)
        this_contact_point = bow_contact_point
        if this_contact_point is None:
            return
        next_contact_point = inspect_(next_leaf).get_indicator(
            indicatortools.BowContactPoint)
        if next_contact_point is None:
            return
        previous_leaf = inspect_(leaf).get_leaf(-1)
        previous_contact_point = None
        if previous_leaf is not None:
            previous_contact_points = inspect_(previous_leaf
                ).get_indicators(indicatortools.BowContactPoint)
            if previous_contact_points:
                previous_contact_point = previous_contact_points[0]
        if (self._is_my_first_leaf(leaf) or
            previous_contact_point is None or
            previous_contact_point.contact_point is None
            ):
            if this_contact_point < next_contact_point:
                direction_change = Down
            elif next_contact_point < this_contact_point:
                direction_change = Up
        else:
            previous_leaf = inspect_(leaf).get_leaf(-1)
            previous_contact_point = inspect_(previous_leaf
                ).get_indicator(indicatortools.BowContactPoint)
            if (previous_contact_point < this_contact_point and
                next_contact_point < this_contact_point):
                direction_change = Up
            elif (this_contact_point < previous_contact_point and
                this_contact_point < next_contact_point):
                direction_change = Down
            elif (this_contact_point == previous_contact_point):
                if this_contact_point < next_contact_point:
                    cautionary_change = True
                    direction_change = Down
                elif next_contact_point < this_contact_point:
                    cautionary_change = True
                    direction_change = Up
        if direction_change is None:
            return
        if cautionary_change:
            if direction_change == Up:
                string = r'^ \parenthesize \upbow'
            elif direction_change == Down:
                string = r'^ \parenthesize \downbow'
        else:
            if direction_change == Up:
                articulation = indicatortools.Articulation('upbow', Up)
            elif direction_change == Down:
                articulation = indicatortools.Articulation('downbow', Up)
            string = str(articulation)
        lilypond_format_bundle.right.articulations.append(string)

    def _make_glissando_overrides(
        self,
        bow_motion_technique=None,
        lilypond_format_bundle=None,
        ):
        if bow_motion_technique is not None:
            style = schemetools.SchemeSymbol(
                bow_motion_technique.glissando_style,
                )
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='Glissando',
                is_once=True,
                property_path='style',
                value=style,
                )
            string = override_.override_string
            lilypond_format_bundle.grob_overrides.append(string)

    def _make_pizzicato_overrides(
        self,
        lilypond_format_bundle=None,
        ):
        style = schemetools.SchemeSymbol('cross')
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='NoteHead',
            is_once=True,
            property_path='style',
            value=style,
            )
        string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(string)

    def _next_leaf_is_bowed(self, leaf):
        if self._is_my_last_leaf(leaf):
            return False
        prototype = (
            scoretools.MultimeasureRest,
            scoretools.Rest,
            scoretools.Skip,
            )
        next_leaf = inspect_(leaf).get_leaf(1)
        if next_leaf is None or isinstance(next_leaf, prototype):
            return False
        next_contact_point = inspect_(next_leaf).get_indicator(
            indicatortools.BowContactPoint)
        if next_contact_point is None:
            return False
        elif next_contact_point.contact_point is None:
            return False
        return True



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Repeat(AbjadValueObject):
    r'''Repeat.

    ..  container:: example

        **Example 1.** Volta repeat:

        ::

            >>> container = Container("c'4 d'4 e'4 f'4")
            >>> repeat = indicatortools.Repeat()
            >>> attach(repeat, container)
            >>> staff = Staff([container])
            >>> score = Score([staff])
            >>> show(score)  # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    \repeat volta 2
                    {
                        c'4
                        d'4
                        e'4
                        f'4
                    }
                }
            >>

    ..  container:: example

        **Example 2.** Unfold repeat:

        ::

            >>> container = Container("c'4 d'4 e'4 f'4")
            >>> repeat = indicatortools.Repeat(repeat_type='unfold')
            >>> attach(repeat, container)
            >>> staff = Staff([container])
            >>> score = Score([staff])
            >>> show(score)  # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    \repeat unfold 2
                    {
                        c'4
                        d'4
                        e'4
                        f'4
                    }
                }
            >>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_repeat_count',
        '_repeat_type',
        )

    _format_leaf_children = False

    _format_slot = 'before'

    ### INITIALIZER ###

    def __init__(self, repeat_count=2, repeat_type='volta'):
        from abjad.tools import scoretools
        # TODO: make score-scoped
        #self._default_scope = scoretools.Score
        self._default_scope = None
        repeat_count = int(repeat_count)
        assert 1 < repeat_count
        self._repeat_count = repeat_count
        assert repeat_type in ('volta', 'unfold')
        self._repeat_type = repeat_type

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of repeat.

        ..  container:: example

            **Example 1.** Volta repeat:

            ::

                >>> str(indicatortools.Repeat())
                '\\repeat volta 2'

        ..  container:: example

            **Example 2.** Unfold repeat:

            ::

                >>> str(indicatortools.Repeat(repeat_type='unfold'))
                '\\repeat unfold 2'

        Returns string.
        '''
        return r'\repeat {} {}'.format(
            self.repeat_type,
            self.repeat_count,
            )

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.before.commands.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of repeat.

        ..  container:: example

            **Example 1.** Volta repeat:

            ::

                >>> repeat = indicatortools.Repeat()
                >>> repeat.default_scope is None
                True

        ..  container:: example

            **Example 2.** Unfold repeat:

            ::

                >>> repeat = indicatortools.Repeat(repeat_type='unfold')
                >>> repeat.default_scope is None
                True

        ..  todo:: Make repeats score-scoped.

        Returns none (but should return score).
        '''
        return self._default_scope

    @property
    def repeat_count(self):
        r'''Gets repeat count of repeat.

        ..  container:: example

            **Example 1.** Volta repeat:

            ::

                >>> repeat = indicatortools.Repeat()
                >>> repeat.repeat_count
                2

        ..  container:: example

            **Example 2.** Unfold repeat:

            ::

                >>> repeat = indicatortools.Repeat(repeat_type='unfold')
                >>> repeat.repeat_count
                2

        Defaults to 2.

        Set to positive integer.

        Returns positive integer.
        '''
        return self._repeat_count

    @property
    def repeat_type(self):
        r'''Gets repeat type of repeat.

        ..  container:: example

            **Example 1.** Volta repeat:

            ::

                >>> repeat = indicatortools.Repeat()
                >>> repeat.repeat_type
                'volta'

        ..  container:: example

            **Example 2.** Unfold repeat:

            ::

                >>> repeat = indicatortools.Repeat(repeat_type='unfold')
                >>> repeat.repeat_type
                'unfold'

        Defaults to ``'volta'``.

        Set to known string.

        Returns string.
        '''
        return self._repeat_type



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.topleveltools import attach


def make_notes_with_multiplied_durations(
    pitch,
    written_duration,
    multiplied_durations,
    ):
    '''Make `written_duration` notes with `pitch` and `multiplied_durations`:

    ::

        >>> arguments = [0, Duration(1, 4), [(1, 2), (1, 3), (1, 4), (1, 5)]]
        >>> scoretools.make_notes_with_multiplied_durations(*arguments)
        Selection([Note("c'4 * 2"), Note("c'4 * 4/3"), Note("c'4 * 1"), Note("c'4 * 4/5")])

    Useful for making spatially positioned notes.

    Returns list of notes.
    '''
    from abjad.tools import scoretools
    from abjad.tools import selectiontools

    # initialize input
    written_duration = durationtools.Duration(written_duration)

    # make notes
    notes = []
    for multiplied_duration in multiplied_durations:
        multiplied_duration = durationtools.Duration(multiplied_duration)
        note = scoretools.Note(pitch, written_duration)
        multiplier = multiplied_duration / written_duration
        attach(multiplier, note)
        notes.append(note)

    # return notes
    notes = selectiontools.Selection(notes)
    return notes



# -*- coding: utf-8 -*-
import collections
import os
from abjad import *
from abjad.tools.rhythmmakertools import *


gallery_division_lists = (
    [
        (4, 16), (5, 16), (1, 2), (2, 12),
        (1, 2), (2, 12), (4, 16), (5, 16),
    ],
    [
        (1, 11), (3, 8), (3, 8), (3, 8),
        (3, 8), (3, 8), (1, 11), (3, 8),
    ],
    [
        (3, 15), (3, 15), (3, 16),
        (5, 24), (5, 24), (5, 16),
        (2, 12), (2, 12), (2, 8),
        (3, 28), (3, 28), (3, 16),
        (1, 9), (1, 9), (1, 8),
    ],
    [
        (9, 16), (1, 5), (9, 16),
        (1, 5), (9, 16), (9, 16),
    ],
    )


def make_configurations_by_class():
    r'''Makes configurations by class.
    '''

    configurations_by_class = collections.OrderedDict()
    pairs = []

    ### NoteRhythmMaker ###

    maker = NoteRhythmMaker()
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        tie_specifier=TieSpecifier(
            tie_across_divisions=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        beam_specifier=BeamSpecifier(
            beam_divisions_together=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            decrease_durations_monotonically=False,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            decrease_durations_monotonically=False,
            ),
        tie_specifier=TieSpecifier(
            tie_across_divisions=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            decrease_durations_monotonically=False,
            ),
        beam_specifier=BeamSpecifier(
            beam_divisions_together=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            ),
        tie_specifier=TieSpecifier(
            tie_across_divisions=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            decrease_durations_monotonically=False,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            decrease_durations_monotonically=False,
            ),
        tie_specifier=TieSpecifier(
            tie_across_divisions=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        beam_specifier=BeamSpecifier(
            beam_each_division=False,
            ),
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        beam_specifier=BeamSpecifier(
            beam_each_division=False,
            ),
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            ),
        tie_specifier=TieSpecifier(
            tie_across_divisions=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        beam_specifier=BeamSpecifier(
            beam_each_division=False,
            ),
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            decrease_durations_monotonically=False,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = NoteRhythmMaker(
        beam_specifier=BeamSpecifier(
            beam_each_division=False,
            ),
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            decrease_durations_monotonically=False,
            ),
        tie_specifier=TieSpecifier(
            tie_across_divisions=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    ### RestRhythmMaker ###

    maker = RestRhythmMaker()
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = RestRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            decrease_durations_monotonically=False,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = RestRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = RestRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            decrease_durations_monotonically=False,
            forbidden_written_duration=Duration(1, 4),
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    ### EvenRunRhythmMaker ###

    maker = EvenRunRhythmMaker()
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = EvenRunRhythmMaker(
        beam_specifier=BeamSpecifier(
            beam_divisions_together=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = EvenRunRhythmMaker(
        beam_specifier=BeamSpecifier(
            beam_each_division=False,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = EvenRunRhythmMaker(
        tie_specifier=TieSpecifier(
            tie_across_divisions=True,
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = EvenRunRhythmMaker(
        duration_spelling_specifier=DurationSpellingSpecifier(
            forbidden_written_duration=Duration(1, 4),
            ),
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    maker = EvenRunRhythmMaker(
        exponent=1,
        )
    pair = (maker, gallery_division_lists)
    pairs.append(pair)

    for pair in pairs:
        maker = pair[0]
        if type(maker) not in configurations_by_class:
            configurations_by_class[type(maker)] = []
        configurations_by_class[type(maker)].append(pair)

    return configurations_by_class


if __name__ == '__main__':
    maker = GalleryMaker()
    configurations_by_class = make_configurations_by_class()
    lilypond_file = maker(configurations_by_class)
    file_path = __file__
    directory = os.path.dirname(file_path)
    file_name = 'gallery.pdf'
    file_path = os.path.join(directory, file_name)
    persist(lilypond_file).as_pdf(file_path, remove_ly=False)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Staff___delitem___01():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.Skip)
    assert isinstance(staff[4], scoretools.FixedDurationTuplet)
    del(staff[0])
    assert len(staff) == 4
    assert isinstance(staff[0], Rest)
    assert isinstance(staff[1], Chord)
    assert isinstance(staff[2], scoretools.Skip)
    assert isinstance(staff[3], scoretools.FixedDurationTuplet)
    del(staff[0])
    assert len(staff) == 3
    assert isinstance(staff[0], Chord)
    assert isinstance(staff[1], scoretools.Skip)
    assert isinstance(staff[2], scoretools.FixedDurationTuplet)
    del(staff[0])
    assert len(staff) == 2
    assert isinstance(staff[0], scoretools.Skip)
    assert isinstance(staff[1], scoretools.FixedDurationTuplet)
    del(staff[0])
    assert len(staff) == 1
    assert isinstance(staff[0], scoretools.FixedDurationTuplet)
    del(staff[0])
    assert len(staff) == 0


def test_scoretools_Staff___delitem___02():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.Skip)
    assert isinstance(staff[4], scoretools.FixedDurationTuplet)
    del(staff[-1])
    assert len(staff) == 4
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.Skip)
    del(staff[-1])
    assert len(staff) == 3
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    assert isinstance(staff[2], Chord)
    del(staff[-1])
    assert len(staff) == 2
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    del(staff[-1])
    assert len(staff) == 1
    assert isinstance(staff[0], Note)
    del(staff[-1])
    assert len(staff) == 0


def test_scoretools_Staff___delitem___03():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.Skip)
    assert isinstance(staff[4], scoretools.FixedDurationTuplet)
    del(staff[3])
    assert len(staff) == 4
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.FixedDurationTuplet)
    del(staff[-2])
    assert len(staff) == 3
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    assert isinstance(staff[2], scoretools.FixedDurationTuplet)
    del(staff[2])
    assert len(staff) == 2
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    del(staff[0])
    assert len(staff) == 1
    assert isinstance(staff[0], Rest)
    del(staff[-1])
    assert len(staff) == 0



# -*- coding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadValueObject


class IndicatorExpression(AbjadValueObject):
    r'''Indicator expression.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Internals'

    __slots__ = (
        '_component',
        '_effective_context',
        '_indicator',
        '_is_annotation',
        '_name',
        '_scope',
        '_synthetic_offset',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        component=None,
        indicator=None,
        is_annotation=None,
        name=None,
        scope=None,
        synthetic_offset=None,
        ):
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        assert not isinstance(indicator, type(self)), repr(indicator)
        if component is not None:
            #assert isinstance(component, scoretools.Component)
            prototype = (scoretools.Component, spannertools.Spanner)
            assert isinstance(component, prototype)
        if scope is not None:
            if isinstance(scope, type):
                assert issubclass(scope, scoretools.Component)
            else:
                assert isinstance(scope, (scoretools.Component, str))
        self._indicator = indicator
        if is_annotation is not None:
            is_annotation = bool(is_annotation)
        self._is_annotation = is_annotation
        self._component = component
        if name is not None:
            name = str(name)
        self._name = name
        self._scope = scope
        self._effective_context = None
        if synthetic_offset is not None:
            synthetic_offset = durationtools.Offset(synthetic_offset)
        self._synthetic_offset = synthetic_offset

    ### SPECIAL METHODS ###

    def __copy__(self):
        r'''Copies indicator expression.

        Note that indicator and scope are copied
        but that start component is not copied.
        This is to avoid start component reference problems.

        Returns new indicator expression.
        '''
        new = type(self)(
            component=None,
            indicator=copy.copy(self.indicator),
            is_annotation=self.is_annotation,
            name=self.name,
            scope=self.scope,
            synthetic_offset=self.synthetic_offset,
            )
        return new

    def __repr__(self):
        '''Gets interpreter representation of indicator expression.

        Returns string.
        '''
        result = '{}({!r}, {!s}, scope={!r}, is_annotation={!r}, name={!r})'
        result = result.format(
            type(self).__name__,
            self.indicator,
            self.component,
            self.scope,
            self.is_annotation,
            self.name,
            )
        return result

    ### PRIVATE METHODS ###

    def _bind_correct_effective_context(self, correct_effective_context):
        from abjad.tools import indicatortools
        self._unbind_effective_context()
        if correct_effective_context is not None:
            correct_effective_context._dependent_expressions.append(self)
        self._effective_context = correct_effective_context
        self._update_effective_context()
        if isinstance(self.indicator, indicatortools.Tempo):
            correct_effective_context._update_later(offsets_in_seconds=True)

    def _bind_to_component(self, component):
        from abjad.tools import indicatortools
        self._warn_duplicate_indicator(component)
        self._unbind_component()
        self._component = component
        self._update_effective_context()
        if isinstance(self.indicator, indicatortools.Tempo):
            self._component._update_later(offsets_in_seconds=True)
        component._indicator_expressions.append(self)

    def _detach(self):
        self._unbind_component()
        self._unbind_effective_context()
        return self

    def _find_correct_effective_context(self):
        scope = self.scope
        if scope is None:
            return None
        elif isinstance(scope, type):
            scope_type = scope
            for component in self.component._get_parentage():
                if isinstance(component, scope_type):
                    return component
        elif isinstance(scope, str):
            scope_name = scope
            for component in self.component._get_parentage():
                if getattr(component, 'context_name', None) == scope_name:
                    return component
        else:
            message = 'target context {!r} must be'
            message += ' context type, context name or none.'
            message = message.format(scope)
            raise TypeError(message)

    def _get_effective_context(self):
        if self.component is not None:
            self.component._update_now(indicators=True)
        return self._effective_context

    def _get_format_pieces(self):
        from abjad.tools import indicatortools
        from abjad.tools import scoretools
        result = []
        if self.is_annotation:
            return result
        if hasattr(self.indicator, '_get_lilypond_format_bundle'):
            return self.indicator._get_lilypond_format_bundle(self.component)
        lilypond_format = self.indicator._get_lilypond_format()
        if isinstance(lilypond_format, (tuple, list)):
            result.extend(lilypond_format)
        else:
            result.append(lilypond_format)
        if self._get_effective_context() is not None:
            return result
        if isinstance(self.indicator, indicatortools.TimeSignature):
            if isinstance(self.component, scoretools.Measure):
                return result
        result = [r'%%% {} %%%'.format(x) for x in result]
        return result

    def _is_formattable_for_component(self, component):
        from abjad.tools import scoretools
        from abjad.tools import indicatortools
        if not hasattr(self.indicator, '_format_slot'):
            return False
        if self.is_annotation:
            return False
        if isinstance(self.component, scoretools.Measure):
            if self.component is component:
                if not isinstance(
                    self.indicator, indicatortools.TimeSignature):
                    return True
                elif component.always_format_time_signature:
                    return True
                else:
                    previous_measure = \
                        scoretools.get_previous_measure_from_component(
                            self.component)
                    if previous_measure is not None:
                        previous_effective_time_signature = \
                            previous_measure.time_signature
                    else:
                        previous_effective_time_signature = None
                    if not self.indicator == previous_effective_time_signature:
                        return True
        elif self.indicator._format_slot == 'right':
            if self.component is component:
                return True
        elif self.component is component:
            return True
        return False

    def _unbind_component(self):
        component = self.component
        if component is not None:
            if hasattr(component, '_indicator_expressions'):
                if self in component._indicator_expressions:
                    component._indicator_expressions.remove(self)
        self._component = None

    def _unbind_effective_context(self):
        effective_context = self._effective_context
        if effective_context is not None:
            try:
                effective_context._dependent_expressions.remove(self)
            except ValueError:
                pass
        self._effective_context = None

    def _update_effective_context(self):
        r'''This function is designed to be called by score components
        during score update.
        '''
        current_effective_context = self._effective_context
        correct_effective_context = self._find_correct_effective_context()
        if current_effective_context is not correct_effective_context:
            self._bind_correct_effective_context(correct_effective_context)

    def _warn_duplicate_indicator(self, component):
        from abjad.tools import spannertools
        if isinstance(component, spannertools.Spanner):
            return
        prototype = type(self.indicator)
        effective = component._get_effective(prototype, unwrap=False)
        if (
            effective is not None and
            effective.scope is not None and
            effective.indicator != self.indicator
            ):
            if effective.start_offset == self.start_offset:
                message = 'effective indicator already attached: {!r}.'
                message = message.format(effective)
                raise ValueError(message)

    ### PUBLIC PROPERTIES ###

    @property
    def component(self):
        r'''Start component of indicator expression.

        Returns component.
        '''
        from abjad.tools import spannertools
        if isinstance(self._component, spannertools.Spanner):
            if self._component:
                return self._component[0]
            else:
                return
        return self._component

    @property
    def indicator(self):
        r'''Indicator of indicator expression.

        Returns indicator.
        '''
        return self._indicator

    @property
    def is_annotation(self):
        r'''Is true if indicator expression is annotative.

        Returns true or false.
        '''
        return self._is_annotation

    @property
    def name(self):
        r'''Indicator expression name.

        Returns string.
        '''
        return self._name

    @property
    def scope(self):
        r'''Target context of indicator expression.

        Returns context.
        '''
        return self._scope

    @property
    def start_offset(self):
        r'''Start offset of indicator expression.

        This is either the expression's synthetic offset or the start offset of
        the expression's component.

        Returns offset.
        '''
        if self._synthetic_offset is not None:
            return self._synthetic_offset
        return self._component._get_timespan().start_offset

    @property
    def synthetic_offset(self):
        r'''Optional synthetic offset.

        Returns offset or None.
        '''
        return self._synthetic_offset



# -*- coding: utf-8 -*-
import os
from abjad import abjad_configuration
from abjad.tools import commandlinetools
from abjad.tools import stringtools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    def side_effect(self, command, **keywords):
        _, file_name = command.split()
        contents = stringtools.normalize('''
        segment_c
        segment_b
        segment_a
        ''')
        with open(file_name, 'w') as file_pointer:
            file_pointer.write(contents)

    @mock.patch('abjad.commandlinetools.ScorePackageScript._call_subprocess')
    def test_success(self, call_subprocess_mock):
        call_subprocess_mock.return_value = 0
        self.create_score()
        self.create_segment('segment_a')
        self.create_segment('segment_b')
        self.create_segment('segment_c')
        script = commandlinetools.ManageSegmentScript()
        command = ['--stage']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
        Staging segments:
            Reading test_score/segments/metadata.json ... OK!
        Staged:
            segment_a
            segment_b
            segment_c
        '''.replace('/', os.path.sep))
        call_subprocess_mock.assert_called_with(
            '{} segments.txt'.format(abjad_configuration.get_text_editor()),
            )
        call_subprocess_mock.side_effect = self.side_effect
        self.reset_string_io()
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
        Staging segments:
            Reading test_score/segments/metadata.json ... OK!
            Writing test_score/segments/metadata.json
        Staged:
            segment_c
            segment_b
            segment_a
        '''.replace('/', os.path.sep))
        call_subprocess_mock.assert_called_with(
            '{} segments.txt'.format(abjad_configuration.get_text_editor()),
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent_is_bar_line_crossing_01():
    r'''Works with partial.
    '''

    staff = Staff("c'8 d'8 e'4 f'8")
    time_signature = TimeSignature((2, 8), partial=Duration(1, 8))
    attach(time_signature, staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \partial 8
            \time 2/8
            c'8
            d'8
            e'4
            f'8
        }
        '''
        )

    assert not inspect_(staff[0]).is_bar_line_crossing()
    assert not inspect_(staff[1]).is_bar_line_crossing()
    assert inspect_(staff[2]).is_bar_line_crossing()
    assert not inspect_(staff[3]).is_bar_line_crossing()


def test_agenttools_InspectionAgent_is_bar_line_crossing_02():
    r'''Works when no explicit time signature is attached.
    '''

    staff = Staff("c'2 d'1 e'2")

    assert not inspect_(staff[0]).is_bar_line_crossing()
    assert inspect_(staff[1]).is_bar_line_crossing()
    assert not inspect_(staff[2]).is_bar_line_crossing()



# -*- coding: utf-8 -*-


def select(argument):
    r'''Select `argument` for tonal analysis.

    Returns tonal analysis selection.
    '''
    from abjad.tools import scoretools
    from abjad.tools import tonalanalysistools
    if isinstance(argument, scoretools.Component):
        return tonalanalysistools.TonalAnalysisAgent(argument)
    elif hasattr(argument, '_music'):
        music = argument._music
        return tonalanalysistools.TonalAnalysisAgent(music)
    else:
        return tonalanalysistools.TonalAnalysisAgent(argument)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordSuspension_figured_bass_string_01():

    chord_suspension = tonalanalysistools.ChordSuspension(4, 3)
    assert chord_suspension.figured_bass_string == '4-3'

    chord_suspension = tonalanalysistools.ChordSuspension(('flat', 2), 1)
    assert chord_suspension.figured_bass_string == 'b2-1'

    chord_suspension = tonalanalysistools.ChordSuspension()
    assert chord_suspension.figured_bass_string == ''



# -*- coding: utf-8 -*-


def timespan_2_starts_when_timespan_1_starts(
    timespan_1=None,
    timespan_2=None,
    hold=False,
    ):
    r'''Makes time relation indicating that `timespan_2` starts
    when `timespan_1` starts.

    ::

        >>> relation = timespantools.timespan_2_starts_when_timespan_1_starts()
        >>> print(format(relation))
        timespantools.TimespanTimespanTimeRelation(
            inequality=timespantools.CompoundInequality(
                [
                    timespantools.Inequality('timespan_1.start_offset == timespan_2.start_offset'),
                    ],
                logical_operator='and',
                ),
            )

    Returns time relation or boolean.
    '''
    from abjad.tools import timespantools

    inequality = timespantools.CompoundInequality([
        'timespan_1.start_offset == timespan_2.start_offset',
        ])

    time_relation = timespantools.TimespanTimespanTimeRelation(
        inequality,
        timespan_1=timespan_1,
        timespan_2=timespan_2,
        )

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



# -*- coding: utf-8 -*-
import copy
from abjad.tools import systemtools
from abjad.tools.datastructuretools.TypedList import TypedList


class Registration(TypedList):
    '''Registration.

    ..  container:: example

        Registration in two parts:

        ::

            >>> components = [('[A0, C4)', 15), ('[C4, C8)', 27)]
            >>> registration = Registration(components)

        ::

            >>> f(registration)
            pitchtools.Registration(
                [
                    pitchtools.RegistrationComponent(
                        source_pitch_range=pitchtools.PitchRange(
                            range_string='[A0, C4)',
                            ),
                        target_octave_start_pitch=pitchtools.NumberedPitch(15),
                        ),
                    pitchtools.RegistrationComponent(
                        source_pitch_range=pitchtools.PitchRange(
                            range_string='[C4, C8)',
                            ),
                        target_octave_start_pitch=pitchtools.NumberedPitch(27),
                        ),
                    ]
                )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __call__(self, pitches):
        r"""Calls registration on `pitches`.

        ..  container:: example

            Transposes four pitches:

            ::

                >>> components = [('[A0, C4)', 15), ('[C4, C8)', 27)]
                >>> registration = Registration(components)
                >>> pitches = registration([-24, -22, -23, -21])
                >>> for pitch in pitches:
                ...     pitch
                NamedPitch("c'''")
                NamedPitch("d'''")
                NamedPitch("cs'''")
                NamedPitch("ef''")

        ..  container:: example

            Transposes four other pitches:

            ::

                >>> components = [('[A0, C4)', 15), ('[C4, C8)', 27)]
                >>> registration = Registration(components)
                >>> pitches = registration([0, 2, 1, 3])
                >>> for pitch in pitches:
                ...     pitch
                NamedPitch("c''''")
                NamedPitch("d''''")
                NamedPitch("cs''''")
                NamedPitch("ef'''")

        ..  container:: example

            Transposes four quartertones:

            ::

                >>> components = [('[A0, C4)', 15), ('[C4, C8)', 27)]
                >>> registration = Registration(components)
                >>> pitches = registration([0.5, 2.5, 1.5, 3.5])
                >>> for pitch in pitches:
                ...     pitch
                NamedPitch("cqs''''")
                NamedPitch("dqs''''")
                NamedPitch("dqf''''")
                NamedPitch("eqf'''")

        Returns list of new pitches.
        """
        transposed_pitches = [self._transpose_pitch(x) for x in pitches]
        return transposed_pitches

    def __format__(self, format_specification=''):
        r'''Formats registration.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            Gets storage format of registration:

            ::

                >>> components = [('[A0, C4)', 15), ('[C4, C8)', 27)]
                >>> registration = Registration(components)

            ::

                >>> f(registration)
                pitchtools.Registration(
                    [
                        pitchtools.RegistrationComponent(
                            source_pitch_range=pitchtools.PitchRange(
                                range_string='[A0, C4)',
                                ),
                            target_octave_start_pitch=pitchtools.NumberedPitch(15),
                            ),
                        pitchtools.RegistrationComponent(
                            source_pitch_range=pitchtools.PitchRange(
                                range_string='[C4, C8)',
                                ),
                            target_octave_start_pitch=pitchtools.NumberedPitch(27),
                            ),
                        ]
                    )

        Returns string.
        '''
        superclass = super(Registration, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE PROPERTIES ###

    @property
    def _item_coercer(self):
        def coerce_(argument):
            if isinstance(argument, tuple):
                component = pitchtools.RegistrationComponent(*argument)
            elif isinstance(argument, pitchtools.RegistrationComponent):
                component = copy.copy(argument)
            else:
                raise TypeError(repr(argument))
            return component
        from abjad.tools import pitchtools
        return coerce_

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = []
        for registration_component in self:
            item = (
                registration_component.source_pitch_range.range_string,
                registration_component.target_octave_start_pitch.pitch_number
                )
            values.append(item)
        return systemtools.FormatSpecification(
            client=self,
            repr_args_values=[values],
            repr_is_indented=False,
            repr_kwargs_names=[],
            storage_format_args_values=[self._collection],
            storage_format_kwargs_names=[],
            )

    def _transpose_pitch(self, pitch):
        from abjad.tools import pitchtools
        pitch = pitchtools.NamedPitch(pitch)
        for component in self:
            if pitch in component.source_pitch_range:
                start_pitch = component.target_octave_start_pitch
                stop_pitch = start_pitch + 12
                if start_pitch <= pitch < stop_pitch:
                    return pitch
                elif pitch < start_pitch:
                    while pitch < start_pitch:
                        pitch += 12
                    return pitch
                elif stop_pitch <= pitch:
                    while stop_pitch <= pitch:
                        pitch -= 12
                    return pitch
                else:
                    raise ValueError(pitch, self)
        else:
            message = 'pitch {} not in {}.'
            message = message.format(pitch, self)
            raise ValueError(message)



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class LilyPondGrobOverride(AbjadObject):
    r'''LilyPond grob override.

    ::

        >>> override = lilypondnametools.LilyPondGrobOverride(
        ...    context_name='Staff',
        ...    grob_name='TextSpanner',
        ...    is_once=True,
        ...    property_path=(
        ...        'bound-details',
        ...        'left',
        ...        'text',
        ...        ),
        ...    value=Markup(r'\bold { over pressure }'),
        ...    )

    ::

        >>> print(override.override_string)
        \once \override Staff.TextSpanner.bound-details.left.text = \markup {
            \bold
                {
                    over
                    pressure
                }
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_context_name',
        '_grob_name',
        '_is_once',
        '_is_revert',
        '_property_path',
        '_value',
        )

    _format_leaf_children = False

    ### INITIALIZER ###

    def __init__(
        self,
        context_name=None,
        grob_name='NoteHead',
        is_once=None,
        is_revert=None,
        property_path='color',
        value='red',
        ):
        if context_name is not None:
            context_name = str(context_name)
        self._context_name = context_name
        assert grob_name
        self._grob_name = str(grob_name)
        if is_once is not None:
            is_once = bool(is_once)
        self._is_once = is_once
        if is_revert is not None:
            is_revert = bool(is_revert)
        self._is_revert = is_revert
        if isinstance(property_path, str):
            property_path = [property_path]
        property_path = tuple(property_path)
        assert all(isinstance(x, str) and x for x in property_path)
        self._property_path = property_path
        self._value = value

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a LilyPond grob override with equivalent
        keyword values.
        '''
        from abjad.tools import systemtools
        return systemtools.TestManager.compare_objects(self, argument)

    def __hash__(self):
        r'''Hashes LilyPond grob override.

        Returns integer.
        '''
        from abjad.tools import systemtools
        hash_values = systemtools.StorageFormatAgent(self).get_hash_values()
        return hash(hash_values)

    ### PRIVATE METHODS ###

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        if not self.is_once:
            revert_format = '\n'.join(self.revert_format_pieces)
            lilypond_format_bundle.grob_reverts.append(revert_format)
        if not self.is_revert:
            override_format = '\n'.join(self.override_format_pieces)
            lilypond_format_bundle.grob_overrides.append(override_format)
        return lilypond_format_bundle

    ### PRIVATE PROPERTIES ###

    @property
    def _override_property_path_string(self):
        parts = []
        if self.context_name is not None:
            parts.append(self.context_name)
        parts.append(self.grob_name)
        parts.extend(self.property_path)
        path = '.'.join(parts)
        return path

    @property
    def _revert_property_path_string(self):
        parts = []
        if self.context_name is not None:
            parts.append(self.context_name)
        parts.append(self.grob_name)
        parts.append(self.property_path[0])
        path = '.'.join(parts)
        return path

    ### PUBLIC PROPERTIES ###

    @property
    def context_name(self):
        r'''Optional LilyPond grob override context name.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...    context_name='Staff',
            ...    grob_name='TextSpanner',
            ...    is_once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=Markup(r'\bold { over pressure }'),
            ...    )
            >>> override.context_name
            'Staff'

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=schemetools.SchemeSymbol('zigzag'),
            ...     )
            >>> override.context_name is None
            True

        Returns string or none.
        '''
        return self._context_name

    @property
    def grob_name(self):
        r'''LilyPond grob override grob name.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=schemetools.SchemeSymbol('zigzag'),
            ...     )
            >>> override.grob_name
            'Glissando'

        Returns string.
        '''
        return self._grob_name

    @property
    def is_once(self):
        r'''Is true if grob override is to be applied only once.  Otherwise
        false.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...    context_name='Staff',
            ...    grob_name='TextSpanner',
            ...    is_once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=Markup(r'\bold { over pressure }'),
            ...    )
            >>> bool(override.is_once)
            True

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=schemetools.SchemeSymbol('zigzag'),
            ...     )
            >>> bool(override.is_once)
            False

        Returns boolean or none.
        '''
        return self._is_once

    @property
    def is_revert(self):
        r'''Is true if grob override is a grob revert. Otherwise false.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=schemetools.SchemeSymbol('zigzag'),
            ...     )
            >>> bool(override.is_revert)
            False

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     is_revert=True,
            ...     property_path='style',
            ...     )
            >>> bool(override.is_revert)
            True

        Returns boolean or none.
        '''
        return self._is_revert

    @property
    def override_format_pieces(self):
        r'''Gets LilyPond grob override \override format pieces.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...    context_name='Staff',
            ...    grob_name='TextSpanner',
            ...    is_once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=Markup(r'\bold { over pressure }'),
            ...    )
            >>> for line in override.override_format_pieces:
            ...     line
            ...
            '\\once \\override Staff.TextSpanner.bound-details.left.text = \\markup {'
            '    \\bold'
            '        {'
            '            over'
            '            pressure'
            '        }'
            '    }'

        Returns tuple of strings.
        '''
        from abjad.tools import schemetools
        result = []
        if self.is_once:
            result.append(r'\once')
        result.append(r'\override')
        result.append(self._override_property_path_string)
        result.append('=')
        value_pieces = schemetools.Scheme.format_embedded_scheme_value(
            self.value)
        value_pieces = value_pieces.split('\n')
        result.append(value_pieces[0])
        result[:] = [' '.join(result)]
        result.extend(value_pieces[1:])
        return tuple(result)

    @property
    def override_string(self):
        r'''Gets LilyPond grob override \override string.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=schemetools.SchemeSymbol('zigzag'),
            ...     )
            >>> override.override_string
            "\\override Glissando.style = #'zigzag"

        Returns string.
        '''
        return '\n'.join(self.override_format_pieces)

    @property
    def property_path(self):
        r'''LilyPond grob override property path.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...    context_name='Staff',
            ...    grob_name='TextSpanner',
            ...    is_once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=Markup(r'\bold { over pressure }'),
            ...    )
            >>> override.property_path
            ('bound-details', 'left', 'text')

        Returns tuple of strings.
        '''
        return self._property_path

    @property
    def revert_format_pieces(self):
        r'''Gets LilyPond grob override \revert format pieces.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=schemetools.SchemeSymbol('zigzag'),
            ...     )
            >>> override.revert_format_pieces
            ('\\revert Glissando.style',)

        Returns tuple of strings.
        '''
        result = r'\revert {}'.format(self._revert_property_path_string)
        return (result,)

    @property
    def revert_string(self):
        r'''Gets LilyPond grob override \revert string.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=schemetools.SchemeSymbol('zigzag'),
            ...     )
            >>> override.revert_string
            '\\revert Glissando.style'

        Returns string.
        '''
        return '\n'.join(self.revert_format_pieces)

    @property
    def value(self):
        r'''Value of LilyPond grob override.

        ::

            >>> override = lilypondnametools.LilyPondGrobOverride(
            ...    context_name='Staff',
            ...    grob_name='TextSpanner',
            ...    is_once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=Markup(r'\bold { over pressure }'),
            ...    )
            >>> override.value
            Markup(contents=[MarkupCommand('bold', ['over', 'pressure'])])

        Returns arbitrary object.
        '''
        return self._value



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Staff_engraver_removals_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    staff.remove_commands.append('Time_signature_engraver')
    staff.remove_commands.append('Bar_number_engraver')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            \remove Time_signature_engraver
            \remove Bar_number_engraver
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitchClass_multiply_01():

    assert pitchtools.NumberedPitchClass(0).multiply(5) == pitchtools.NumberedPitchClass(0)
    assert pitchtools.NumberedPitchClass(1).multiply(5) == pitchtools.NumberedPitchClass(5)
    assert pitchtools.NumberedPitchClass(2).multiply(5) == pitchtools.NumberedPitchClass(10)
    assert pitchtools.NumberedPitchClass(3).multiply(5) == pitchtools.NumberedPitchClass(3)
    assert pitchtools.NumberedPitchClass(4).multiply(5) == pitchtools.NumberedPitchClass(8)
    assert pitchtools.NumberedPitchClass(5).multiply(5) == pitchtools.NumberedPitchClass(1)
    assert pitchtools.NumberedPitchClass(6).multiply(5) == pitchtools.NumberedPitchClass(6)
    assert pitchtools.NumberedPitchClass(7).multiply(5) == pitchtools.NumberedPitchClass(11)
    assert pitchtools.NumberedPitchClass(8).multiply(5) == pitchtools.NumberedPitchClass(4)
    assert pitchtools.NumberedPitchClass(9).multiply(5) == pitchtools.NumberedPitchClass(9)
    assert pitchtools.NumberedPitchClass(10).multiply(5) == pitchtools.NumberedPitchClass(2)
    assert pitchtools.NumberedPitchClass(11).multiply(5) == pitchtools.NumberedPitchClass(7)



# -*- coding: utf-8 -*-
import collections
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Transposition(AbjadValueObject):
    r'''Transposition operator.

    ..  container:: example

        ::

            >>> Transposition()
            Transposition(n=0)

    ..  container:: example

        ::

            >>> Transposition(n=2)
            Transposition(n=2)

    Object model of twelve-tone transposition operator.
    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        '_n',
        )

    ### INITIALIZER ###

    def __init__(self, n=0):
        self._n = n

    ### SPECIAL METHODS ###

    def __add__(self, operator):
        r'''Composes transposition and `operator`.

        ..  container:: example

            Example segment:

            ::

                >>> items = [0, 2, 4, 5]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP
    
            Example operators:

            ::

                >>> T_1 = Transposition(n=1)
                >>> T_3 = Transposition(n=3)

        ..  container:: example

            Successive transposition:

            ::

                >>> operator = T_1 + T_3
                >>> str(operator)
                'T1T3'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    e'8
                    fs'8
                    af'8
                    a'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Same as above because transposition commutes:

            ::

                >>> operator = T_3 + T_1
                >>> str(operator)
                'T3T1'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    e'8
                    fs'8
                    af'8
                    a'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        Returns compound operator.
        '''
        from abjad.tools import pitchtools
        return pitchtools.CompoundOperator._compose_operators(self, operator)

    def __call__(self, argument):
        r'''Calls transposition on `argument`.

        ..  container:: example

            Transposes pitch-class:

            ::

                >>> transposition = Transposition(n=2)
                >>> pitch_class = NumberedPitchClass(1)
                >>> transposition(pitch_class)
                NumberedPitchClass(3)

        ..  container:: example

            Transposes pitch:

            ::

                >>> transposition = Transposition(n=2)
                >>> pitch = NumberedPitch(15)
                >>> transposition(pitch)
                NumberedPitch(17)

        ..  container:: example

            Transposes list of pitches:

            ::

                >>> transposition = Transposition(n=2)
                >>> pitches = [NumberedPitch(_) for _ in [15, 16]]
                >>> transposition(pitches)
                [NumberedPitch(17), NumberedPitch(18)]

        Returns new object with type equal to that of `argument`.
        '''
        if hasattr(argument, 'transpose'):
            result = argument.transpose(self.n)
        elif isinstance(argument, collections.Iterable):
            items = []
            for item in argument:
                item = item.transpose(self.n)
                items.append(item)
            result = type(argument)(items)
        else:
            message = 'do not know how to transpose: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        return result

    def __str__(self):
        r'''Gets string representation of operator.

        ..  container:: example

            ::

                >>> str(Transposition())
                'T0'

        ..  container:: example

            ::

                >>> str(Transposition(n=2))
                'T2'

        '''
        string = 'T{}'
        string = string.format(self.n)
        return string

    ### PRIVATE METHODS ###

    def _get_markup(self, direction=None):
        from abjad.tools import markuptools
        operator = markuptools.Markup('T', direction=None)
        subscript = markuptools.Markup(self.n).sub()
        markup = markuptools.Markup.concat([operator, subscript])
        return markup

    def _is_identity_operator(self):
        if self.n == 0:
            return True
        return False
        
    ### PUBLIC PROPERTIES ###

    @property
    def n(self):
        r'''Gets index of transposition.

        ..  container:: example

            ::

                >>> transposition = Transposition()
                >>> transposition.n
                0

        ..  container:: example

            ::

                >>> transposition = Transposition(n=2)
                >>> transposition.n
                2

        Set to integer, interval or none.
        '''
        return self._n



# -*- coding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.pitchtools.Set import Set


class IntervalClassSet(Set):
    r'''Interval-class set.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        from abjad.tools import pitchtools
        prototype = (
            pitchtools.PitchClassSegment,
            pitchtools.PitchClassSet,
            pitchtools.PitchSegment,
            pitchtools.PitchSet,
            )
        if isinstance(items, prototype):
            items = list(items)
            enumeration = sequencetools.Enumeration(items)
            pairs = enumeration.yield_pairs()
            items = [second - first for first, second in pairs]
        Set.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedIntervalClass

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedIntervalClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.IntervalClass

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(class_, selection, item_class=None):
        r'''Initialize interval set from component selection:

        ::

            >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = Staff("c4. r8 g2")
            >>> selection = select((staff_1, staff_2))
            >>> interval_classes = pitchtools.IntervalClassSet.from_selection(
            ...     selection)
            >>> for interval_class in sorted(interval_classes):
            ...     interval_class
            ...
            NamedIntervalClass('-M6')
            NamedIntervalClass('-P5')
            NamedIntervalClass('-aug4')
            NamedIntervalClass('-M3')
            NamedIntervalClass('-m3')
            NamedIntervalClass('-M2')
            NamedIntervalClass('+m2')
            NamedIntervalClass('+M2')
            NamedIntervalClass('+m3')
            NamedIntervalClass('+M3')
            NamedIntervalClass('+P4')
            NamedIntervalClass('+aug4')
            NamedIntervalClass('+P5')
            NamedIntervalClass('+M6')
            NamedIntervalClass('+m7')
            NamedIntervalClass('+M7')
            NamedIntervalClass('+P8')

        Returns interval set.
        '''
        from abjad.tools import pitchtools
        interval_set = pitchtools.IntervalSet.from_selection(selection)
        return class_(
            items=interval_set,
            item_class=item_class,
            )



# -*- coding: utf-8 -*-
try:
    import ConfigParser as configparser
except ImportError:
    import configparser
from abjad.tools import commandlinetools


class AbjadBookScript(commandlinetools.CommandlineScript):
    r'''Entry point script for abjad-book.

    ::

        >>> from abjad.tools import abjadbooktools
        >>> script = abjadbooktools.AbjadBookScript()
        >>> print(script.formatted_help)
        usage: abjad-book [-h] [--version] [-c] [-o OUTPUT_FILE_PATH] [-s] [-v]
                          [-y STYLESHEET] [-a ASSETS_DIRECTORY]
                          [-l LATEX_ROOT_DIRECTORY] [-g CONFIG]
                          input_file_path [input_file_path ...]
        <BLANKLINE>
        Preprocess LaTeX files with Abjad.
        <BLANKLINE>
        positional arguments:
          input_file_path       LaTeX file to process
        <BLANKLINE>
        optional arguments:
          -h, --help            show this help message and exit
          --version             show program's version number and exit
          -c, --clean           remove all output blocks
          -o OUTPUT_FILE_PATH, --output-file-path OUTPUT_FILE_PATH
                                optional output file path
          -s, --skip-rendering  skip all image rendering and simply execute the code
          -v, --verbose         verbose output
          -y STYLESHEET, --stylesheet STYLESHEET
                                optional LilyPond stylesheet
          -a ASSETS_DIRECTORY, --assets-directory ASSETS_DIRECTORY
                                optional assets directory
          -l LATEX_ROOT_DIRECTORY, --latex-root-directory LATEX_ROOT_DIRECTORY
                                optional LaTeX root directory
          -g CONFIG, --config CONFIG
                                path to config file
        <BLANKLINE>

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Entry Points'

    alias = 'book'
    short_description = 'Preprocess LaTeX files with Abjad.'
    version = 3.0

    ### PRIVATE METHODS ###

    def _process_args(self, arguments):
        from abjad.tools import abjadbooktools
        assets_directory = arguments.assets_directory
        clean = arguments.clean
        configuration = self._read_config(arguments.config)
        input_file_path = arguments.input_file_path
        latex_root_directory = arguments.latex_root_directory
        output_file_path = arguments.output_file_path
        skip_rendering = arguments.skip_rendering
        stylesheet = arguments.stylesheet
        verbose = arguments.verbose
        if 1 < len(input_file_path):
            document_handlers = []
            for path in input_file_path:
                document_handler = abjadbooktools.LaTeXDocumentHandler.from_path(
                    input_file_path=path,
                    assets_directory=assets_directory,
                    latex_root_directory=latex_root_directory,
                    )
                document_handlers.append(document_handler)
            for document_handler in document_handlers:
                document_handler(
                    clean=clean,
                    configuration=configuration,
                    skip_rendering=skip_rendering,
                    stylesheet=stylesheet,
                    verbose=verbose,
                    )
        else:
            document_handler = abjadbooktools.LaTeXDocumentHandler.from_path(
                input_file_path=input_file_path[0],
                assets_directory=assets_directory,
                latex_root_directory=latex_root_directory,
                )
            document_handler(
                clean=clean,
                configuration=configuration,
                output_file_path=output_file_path,
                skip_rendering=skip_rendering,
                stylesheet=stylesheet,
                verbose=verbose,
                )

    def _read_config(self, config_path):
        configuration = {}
        if config_path is None:
            return configuration
        parser = configparser.ConfigParser()
        parser.read(config_path)
        for section in parser.sections():
            configuration[section] = {}
            for key, value in parser.items(section):
                value = tuple(_.strip() for _ in value.splitlines()
                    if _.strip())
                configuration[section][key] = value
        return configuration

    def _setup_argument_parser(self, parser):
        parser.add_argument(
            'input_file_path',
            nargs='+',
            help='LaTeX file to process',
            )
        parser.add_argument(
            '-c', '--clean',
            action='store_true',
            help='remove all output blocks',
            )
        parser.add_argument(
            '-o', '--output-file-path',
            help='optional output file path',
            )
        parser.add_argument(
            '-s', '--skip-rendering',
            action='store_true',
            help='skip all image rendering and simply execute the code',
            )
        parser.add_argument(
            '-v', '--verbose',
            action='store_true',
            help='verbose output',
            )
        parser.add_argument(
            '-y', '--stylesheet',
            help='optional LilyPond stylesheet',
            )
        parser.add_argument(
            '-a', '--assets-directory',
            help='optional assets directory',
            )
        parser.add_argument(
            '-l', '--latex-root-directory',
            help='optional LaTeX root directory',
            )
        parser.add_argument(
            '-g', '--config',
            help='path to config file',
            )



# -*- coding: utf-8 -*-
from abjad.tools import selectiontools


def make_repeated_rests_from_time_signatures(time_signatures):
    r'''Make repeated rests from `time_signatures`:

    ..  container:: example

        ::

            scoretools.make_repeated_rests_from_time_signatures([(2, 8), (3, 32)])
            [[Rest('r8'), Rest('r8')], [Rest('r32'), Rest('r32'), Rest('r32')]]

    Returns two-dimensional list of newly constructed rest lists.
    '''
    from abjad.tools import scoretools

    # initialize result
    result = []

    # iterate time signatures and make rests
    for time_signature in time_signatures:
        rests = _make_repeated_rests_from_time_signature(time_signature)
        result.append(rests)

    # return result
    return result


def _make_repeated_rests_from_time_signature(time_signature):
    from abjad.tools import indicatortools
    from abjad.tools import scoretools

    # afford basic input polymorphism
    time_signature = indicatortools.TimeSignature(time_signature)

    # check input
    if time_signature.has_non_power_of_two_denominator:
        message = 'TODO: extend this function for time signatures'
        message += ' with non-power-of-two denominators.'
        raise NotImplementedError(message)

    # make and return repeated rests
    rest = scoretools.Rest((1, time_signature.denominator))
    result = time_signature.numerator * rest

    result = selectiontools.Selection(result)
    return result



# -*- coding: utf-8 -*-
from __future__ import print_function
import os
import webbrowser
from abjad.tools import systemtools
from abjad.tools.documentationtools import DocumentationManager
from abjad.tools.commandlinetools.CommandlineScript import CommandlineScript


class BuildApiScript(CommandlineScript):
    r'''Builds the Abjad APIs.

    ..  shell::

        ajv api --help

    '''

    ### CLASS VARIABLES ###

    class ExperimentalDocumentationManager(DocumentationManager):
        r'''API generator for the experimental package.
        '''
        api_directory_name = None
        api_title = 'Abjad Experimental API'
        root_package_name = 'experimental'
        source_directory_path_parts = ('docs', 'source')
        tools_packages_package_path = 'experimental.tools'

    class IDEDocumentationManager(DocumentationManager):
        r'''API generator for the Abjad IDE package.
        '''
        api_directory_name = None
        api_title = 'Abjad IDE API'
        root_package_name = 'ide'
        source_directory_path_parts = ('docs', 'source')
        tools_packages_package_path = 'ide.tools'

    class ScoreLibraryDocumentationManager(DocumentationManager):
        r'''API generator for score library documentation.
        '''
        api_directory_name = None
        root_package_name = None
        source_directory_path_parts = None
        tools_packages_package_path = None

        def __init__(
            self,
            api_title,
            docs_directory,
            packages_to_document,
            ):
            self.api_title = api_title
            self.docs_directory = docs_directory
            self.packages_to_document = packages_to_document

    alias = 'api'
    short_description = 'Build the Abjad APIs.'

    ### PRIVATE METHODS ###

    def _build_api(
        self,
        docs_directory=None,
        api_generator=None,
        api_title=None,
        api_format='html',
        clean=False,
        rst_only=False,
        ):
        api_generator.execute()
        if rst_only:
            return
        message = 'Now building the {} {} docs ...'
        message = message.format(api_title, api_format.upper())
        print(message)
        with systemtools.TemporaryDirectoryChange(docs_directory):
            if clean:
                print('Cleaning build directory ...')
                command = 'make clean'
                systemtools.IOManager.spawn_subprocess(command)
            if format == 'coverage':
                command = 'sphinx-build -b coverage {} {}'.format(
                    'source',
                    os.path.join('build', 'coverage'),
                    )
                systemtools.IOManager.spawn_subprocess(command)
            else:
                command = 'make {}'.format(api_format)
                systemtools.IOManager.spawn_subprocess(command)

    def _build_experimental_api(
        self,
        api_format='html',
        clean=False,
        rst_only=False,
        ):
        from abjad import abjad_configuration
        api_generator = BuildApiScript.ExperimentalDocumentationManager()
        api_title = 'experimental'
        docs_directory = os.path.join(
            abjad_configuration.abjad_root_directory,
            'experimental',
            'docs',
            )
        self._build_api(
            api_generator=api_generator,
            api_title=api_title,
            api_format=api_format,
            clean=clean,
            docs_directory=docs_directory,
            rst_only=rst_only,
            )
        path = os.path.join(
            abjad_configuration.abjad_root_directory,
            'experimental',
            'docs',
            'build',
            'html',
            'index.html',
            )
        return path

    def _build_ide_api(
        self,
        api_format='html',
        clean=False,
        rst_only=False,
        ):
        import ide
        api_generator = BuildApiScript.IDEDocumentationManager()
        api_title = 'Abjad IDE'
        docs_directory = os.path.join(
            ide.__path__[0],
            'docs',
            )
        self._build_api(
            api_generator=api_generator,
            api_title=api_title,
            api_format=api_format,
            clean=clean,
            docs_directory=docs_directory,
            rst_only=rst_only,
            )
        path = os.path.join(
            ide.__path__[0],
            'docs',
            'build',
            'html',
            'index.html',
            )
        return path

    def _build_mainline_api(
        self,
        api_format='html',
        clean=False,
        rst_only=False,
        ):
        from abjad import abjad_configuration
        api_generator = DocumentationManager()
        api_title = 'mainline'
        docs_directory = os.path.join(
            abjad_configuration.abjad_directory,
            'docs',
            )
        self._build_api(
            api_generator=api_generator,
            api_title=api_title,
            api_format=api_format,
            clean=clean,
            docs_directory=docs_directory,
            rst_only=rst_only,
            )
        path = os.path.join(
            abjad_configuration.abjad_root_directory,
            'abjad',
            'docs',
            'build',
            'html',
            'api',
            'index.html',
            )
        return path

    def _build_score_library_api(
        self,
        api_title,
        docs_directory,
        packages_to_document,
        api_format='html',
        clean=False,
        rst_only=False,
        ):
        api_generator = BuildApiScript.ScoreLibraryDocumentationManager(
            api_title,
            docs_directory,
            packages_to_document,
            )
        self._build_api(
            api_generator=api_generator,
            api_title=api_title,
            api_format=api_format,
            clean=clean,
            docs_directory=docs_directory,
            rst_only=rst_only,
            )
        path = os.path.join(
            docs_directory,
            'build',
            'html',
            'index.html',
            )
        return path

    def _process_args(self, arguments):
        api_format = arguments.format
        clean = arguments.clean
        rst_only = arguments.rst_only
        paths = []
        prototype = (
            arguments.experimental,
            arguments.ide,
            arguments.mainline,
            arguments.score_library,
            )
        if not any(prototype):
            arguments.mainline = True
        if arguments.experimental:
            path = self._build_experimental_api(
                api_format=api_format,
                clean=clean,
                rst_only=rst_only,
                )
            paths.append(path)
        if arguments.ide:
            path = self._build_ide_api(
                api_format=api_format,
                clean=clean,
                rst_only=rst_only,
                )
            paths.append(path)
        if arguments.mainline:
            path = self._build_mainline_api(
                api_format=api_format,
                clean=clean,
                rst_only=rst_only,
                )
            paths.append(path)
        if arguments.score_library:
            messages = [
                'Must specify all of ...',
                '    --api-title',
                '    --docs-directory',
                '    --packages-to-document',
                '... when building -S or --score-library.',
                ]
            if not all((
                arguments.api_title,
                arguments.docs_directory,
                arguments.packages_to_document,
                )):
                for message in messages:
                    print(message)
                return
            path = self._build_score_library_api(
                arguments.api_title,
                arguments.docs_directory,
                arguments.packages_to_document,
                api_format=api_format,
                clean=clean,
                rst_only=rst_only,
                )
        if api_format == 'html' and arguments.openinbrowser and not rst_only:
            for path in paths:
                if path.startswith('/'):
                    path = 'file://' + path
                webbrowser.open(path)

    def _setup_argument_parser(self, parser):
        parser.add_argument(
            '-C',
            '--clean',
            action='store_true',
            dest='clean',
            help='run "make clean" before building the api',
            )
        parser.add_argument(
            '-I',
            '--ide',
            action='store_true',
            help='build the Abjad IDE API',
            )
        parser.add_argument(
            '-M',
            '--mainline',
            action='store_true',
            help='build the mainline API',
            )
        parser.add_argument(
            '-O',
            '--open',
            action='store_true',
            dest='openinbrowser',
            help='open the docs in a web browser after building',
            )
        parser.add_argument(
            '-R',
            '--rst-only',
            action='store_true',
            dest='rst_only',
            help='generate the ReSt source files but do not build',
            )
        parser.add_argument(
            '-S',
            '--score-library',
            action='store_true',
            help='build score library API',
            )
        parser.add_argument(
            '-X',
            '--experimental',
            action='store_true',
            help='build the experimental API',
            )
        parser.add_argument(
            '--api-title',
            help='title of API',
            ),
        parser.add_argument(
            '--docs-directory',
            help='documentation directory',
            ),
        parser.add_argument(
            '--packages-to-document',
            help='comma-delimited packages list (ex: "foo.tools,bar.tools")',
            ),
        parser.add_argument(
            '--format',
            choices=(
                'coverage',
                'html',
                'latex',
                'latexpdf',
                ),
            help='Sphinx builder to use',
            metavar='FORMAT',
            )
        parser.set_defaults(format='html')
